package org.processmining.timebasedabstraction.algorithms;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import org.apache.commons.collections.MultiMap;
import org.apache.commons.collections.map.MultiValueMap;

import weka.clusterers.DBSCAN;
import weka.core.Attribute;
import weka.core.AttributeStats;
import weka.core.DenseInstance;
import weka.core.EuclideanDistance;
import weka.core.Instance;
import weka.core.Instances;
import weka.experiment.Stats;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.Remove;

public class Dbscan {

	private DBSCAN dbscan = new DBSCAN();
	private Set<Integer> attributeSet;
	private ArrayList<Double> avgSums;
	private ArrayList<ArrayList<Double>> list;
	private Map<Integer, Map<String, Double>> attributeToClusterMap;
	private Map<Integer, Integer> indexToClusterMap; // Mapping of which instance belongs to which cluster
	private Map<Integer, Instance> clusterCentroidMap; // Which centroid belongs to which cluster;
	private MultiMap clusterToInstanceMap; // Mapping of which cluster contains which instances;
	private SortedSet<Integer> indexToNoise;
	private SortedSet<String> attributeNames;
	private SortedSet<Integer> clusters;
	private double actualThreshold;
	private double[][] dataMatrix;
	private double maxValue = 0;
	private double minValue = Double.MAX_VALUE;
	private int numberOfClusters;
	private double epsilon;
	private int minPoints;
	private Instances data;
	private EuclideanDistance Dist;

	public void buildIt(Instances data) throws Exception {
		data.randomize(new Random());
		setData(data);
		dbscan.setEpsilon(getEpsilon());
		dbscan.setMinPoints(getMinPoints());
		dbscan.buildClusterer(data);

		System.out.println(dbscan.getEpsilon());
		System.out.println(dbscan.getMinPoints());


		this.setNumberOfClusters(dbscan.numberOfClusters());

		for (int j = 0; j < data.numAttributes(); j++) {
			AttributeStats attStats = data.attributeStats(j);
			Stats stats = attStats.numericStats;
			double sum = stats.sum;
			double avgSum = sum / data.numInstances();
			if (avgSum > maxValue) {
				maxValue = avgSum;
			}
			if (avgSum < minValue) {
				minValue = avgSum;
			}
		}

		attributeToClusterMap = new TreeMap<Integer, Map<String, Double>>();
		indexToClusterMap = new TreeMap<Integer, Integer>();
		clusterToInstanceMap = new MultiValueMap();
		for (int i = 0; i < data.numInstances(); i++) {
			Instance instance = data.get(i);					
			try {
				Integer clusterAssignment = dbscan.clusterInstance(instance);
				if (!attributeToClusterMap.containsKey(clusterAssignment)) {
					attributeToClusterMap.put(clusterAssignment, new TreeMap<String, Double>());
				}
				indexToClusterMap.put(i, clusterAssignment);
				clusterToInstanceMap.put(clusterAssignment, instance);
			} catch (Exception e) {
				continue;
			}
		}
		
//		calcCentroids();
//		getNoisePoints();
//		assignNoiseToCluster();

		for (Integer key : attributeToClusterMap.keySet()) {
			Map<String, Double> m = attributeToClusterMap.get(key);
			for (int i = 0; i < data.numInstances(); i++) {
				Instance instance = data.get(i);
				try {
					Integer clusterAssignment = dbscan.clusterInstance(instance);
					for (int j = 0; j < data.numAttributes(); j++) {
						Attribute attribute = instance.attribute(j);
						double instanceValue = instance.value(attribute);
						if (clusterAssignment == key) {
							if (!m.containsKey(attribute.name())) {
								m.put(attribute.name(), instanceValue);
							} else {
								m.put(attribute.name(), m.get(attribute.name()) + instanceValue);
							}
						}
					}
				} catch (Exception e) {
					continue;
				}
			}
		}

		attributeNames = new TreeSet<String>();
		for (Integer key : attributeToClusterMap.keySet()) {
			for (String attNames : attributeToClusterMap.get(key).keySet()) {
				attributeNames.add(attNames);
			}
		}

		clusters = new TreeSet<Integer>();
		for (int i = 0; i < data.numInstances(); i++) {
			Instance instance = data.get(i);
			try {
				Integer clusterAssignment = dbscan.clusterInstance(instance);
				clusters.add(clusterAssignment);
			} catch (Exception e) {
				continue;
			}
		}

		list = new ArrayList<ArrayList<Double>>();
		for (String attname : attributeNames) {
			ArrayList<Double> values = new ArrayList<Double>();
			for (Integer key : attributeToClusterMap.keySet()) {
				values.add(attributeToClusterMap.get(key).get(attname));
			}
			list.add(values);
		}

		// Create the matrix (simple way)
		dataMatrix = new double[list.size()][];
		for (int i = 0; i < list.size(); i++) {
			ArrayList<Double> row = list.get(i);
			double[] copy = new double[row.size()];
			for (int j = 0; j < row.size(); j++) {
				copy[j] = row.get(j);
			}
			dataMatrix[i] = copy;
		}

	}

	public void runFiltered(Instances data) throws Exception {
		attributeSet = new HashSet<Integer>();
		for (int j = 0; j < data.numAttributes(); j++) {
			Attribute att = data.attribute(j);
			AttributeStats attStats = data.attributeStats(j);
			Stats stats = attStats.numericStats;
			double sum = stats.sum;
			double avgSum = sum / data.numInstances();

			if (avgSum < getActualThresholdValue()) {
				attributeSet.add(att.index());
			}
		}

		Remove filter = new Remove();
		int[] attributeIndex = new int[attributeSet.size()];
		for (int i = 0; i < attributeIndex.length; i++) {
			for (int value : attributeSet) {
				attributeIndex[i++] = value;
			}
		}
		filter.setAttributeIndicesArray(attributeIndex);
		filter.setInputFormat(data);
		Instances filteredDataSet = Filter.useFilter(data, filter);

		attributeToClusterMap = new TreeMap<Integer, Map<String, Double>>();
		for (int i = 0; i < filteredDataSet.numInstances(); i++) {
			Instance instance = filteredDataSet.get(i);
			try {
				Integer clusterAssignment = dbscan.clusterInstance(instance);
				if (!attributeToClusterMap.containsKey(clusterAssignment)) {
					attributeToClusterMap.put(clusterAssignment, new TreeMap<String, Double>());
				}
			} catch (Exception e) {
				continue;
			}
		}

		for (Integer key : attributeToClusterMap.keySet()) {
			Map<String, Double> m = attributeToClusterMap.get(key);
			for (int i = 0; i < filteredDataSet.numInstances(); i++) {
				Instance instance = filteredDataSet.get(i);
				try {
					Integer clusterAssignment = dbscan.clusterInstance(instance);
					for (int j = 0; j < filteredDataSet.numAttributes(); j++) {
						Attribute attribute = instance.attribute(j);
						double instanceValue = instance.value(attribute);
						if (clusterAssignment == key) {
							if (!m.containsKey(attribute.name())) {
								m.put(attribute.name(), instanceValue);
							} else {
								m.put(attribute.name(), m.get(attribute.name()) + instanceValue);
							}
						}
					}
				} catch (Exception e) {
					continue;
				}
			}
		}

		attributeNames = new TreeSet<String>();
		for (Integer key : attributeToClusterMap.keySet()) {
			for (String attNames : attributeToClusterMap.get(key).keySet()) {
				attributeNames.add(attNames);
			}
		}

		clusters = new TreeSet<Integer>();
		for (int i = 0; i < filteredDataSet.numInstances(); i++) {
			Instance instance = filteredDataSet.get(i);
			try {
				Integer clusterAssignment = dbscan.clusterInstance(instance);
				clusters.add(clusterAssignment);
			} catch (Exception e) {
				continue;
			}
		}

		list = new ArrayList<ArrayList<Double>>();
		for (String attname : attributeNames) {
			ArrayList<Double> values = new ArrayList<Double>();
			for (Integer key : attributeToClusterMap.keySet()) {
				values.add(attributeToClusterMap.get(key).get(attname));
			}
			list.add(values);
		}

		// Create the matrix (simple way)
		dataMatrix = new double[list.size()][];
		for (int i = 0; i < list.size(); i++) {
			ArrayList<Double> row = list.get(i);
			double[] copy = new double[row.size()];
			for (int j = 0; j < row.size(); j++) {
				copy[j] = row.get(j);
			}
			dataMatrix[i] = copy;
		}
		
//		calcCentroids();
//		getNoisePoints();
//		assignNoiseToCluster();

	}

	public void calcCentroids() {
		clusterCentroidMap = new TreeMap<Integer, Instance>();
		Set<Integer> keys = clusterToInstanceMap.keySet();
		for (Integer keyprint : keys) {
			Collection<Instance> vals = (Collection<Instance>) clusterToInstanceMap.get(keyprint);
			double[] instanceArray = new double[getData().numAttributes()];
			for (int i = 0; i < getData().numAttributes(); i++) {
				double sum = 0;
				for (Instance inst : vals) {
					Attribute att = inst.attribute(i);
					double value = inst.value(att);
					sum += value;
				}
				double avg = sum / vals.size();
				instanceArray[i] = avg;
			}
			Instance centroid = new DenseInstance(1, instanceArray);
			System.out.println("cluster:" + keyprint + " , " + "centroid:" + centroid);
			clusterCentroidMap.put(keyprint, centroid);
		}
	}

	public void getNoisePoints() {
		Map<Integer, Integer> clusterAssignments = getClusterAssignments();
		indexToNoise = new TreeSet<Integer>();
		for (int i = 0; i < clusterAssignments.size(); i++) {
			if (clusterAssignments.get(i) == null) {
				System.out.println("noise added!");
				indexToNoise.add(i);
				
			}
		}
		System.out.println(indexToNoise);

	}

	public void assignNoiseToCluster() {
		Dist = new EuclideanDistance(getData());

		for (Integer i : indexToNoise) {
			double dist = Double.MAX_VALUE;
			Integer clust = 0;
			System.out.println("dist initial: " + dist);
			for (Integer cluster : clusterCentroidMap.keySet()) {
				Instance centroid = clusterCentroidMap.get(cluster);
				Double distance = Dist.distance(getData().get(i), centroid);

				if (distance < dist) {
					dist = distance;
					clust = cluster;
					System.out.println(clust);
				}
				System.out.println("dist: " + dist);
				this.indexToClusterMap.put(i, clust);				
			}
		}
	}

	public SortedSet<Integer> getClusters() {
		return clusters;
	}

	public void setData(Instances data) {
		this.data = data;
	}

	public Instances getData() {
		return this.data;
	}

	public SortedSet<String> getAttributeNames() {
		return attributeNames;
	}

	public double[][] getMatrix() {
		return dataMatrix;
	}

	public void setNumberOfClusters(int numberOfClusters) {
		this.numberOfClusters = numberOfClusters;
	}

	public int getNumberOfClusters() {
		return numberOfClusters;
	}

	public void setActualThresholdValue(double value) {
		this.actualThreshold = value;
	}

	public double getActualThresholdValue() {
		return actualThreshold;
	}

	public double getMinValue() {
		return minValue;
	}

	public double getMaxValue() {
		return maxValue;
	}

	public Map<Integer, Map<String, Double>> getClusterToAttributeValueMap() {
		return attributeToClusterMap;
	}

	public void setEpsilon(double epsilon) {
		this.epsilon = epsilon;
	}

	public double getEpsilon() {
		return epsilon;
	}

	public void setMinPoints(int minPoints) {
		this.minPoints = minPoints;
	}

	public int getMinPoints() {
		return minPoints;
	}

	public Map<Integer, Integer> getClusterAssignments() {
		return indexToClusterMap;
	}

}
