package org.processmining.timebasedabstraction.algorithms;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import org.apache.commons.math3.stat.correlation.PearsonsCorrelation;

import de.lmu.ifi.dbs.elki.algorithm.clustering.DBSCAN;
import de.lmu.ifi.dbs.elki.data.Cluster;
import de.lmu.ifi.dbs.elki.data.Clustering;
import de.lmu.ifi.dbs.elki.data.DoubleVector;
import de.lmu.ifi.dbs.elki.data.NumberVector;
import de.lmu.ifi.dbs.elki.data.model.Model;
import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
import de.lmu.ifi.dbs.elki.database.Database;
import de.lmu.ifi.dbs.elki.database.StaticArrayDatabase;
import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
import de.lmu.ifi.dbs.elki.database.ids.DBIDMIter;
import de.lmu.ifi.dbs.elki.database.ids.DBIDRange;
import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
import de.lmu.ifi.dbs.elki.database.relation.Relation;
import de.lmu.ifi.dbs.elki.datasource.ArrayAdapterDatabaseConnection;
import de.lmu.ifi.dbs.elki.datasource.DatabaseConnection;
import de.lmu.ifi.dbs.elki.utilities.ClassGenericsUtil;
import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.ListParameterization;

public class DbscanElki {
	private double[][] data;
	private double[][] dataMatrix;
	private double[][] oldDataMatrix;

	private int minPoints;
	private double epsilon;
	private int numberOfClusters;
	private String[] labels;
	private boolean isNoise;
	private int noiseCluster;

	private double maxValue = 0;
	private double minValue = Double.MAX_VALUE;

	private Map<Integer, NumberVector> clusterCentroidMap;
	private Map<Integer, double[]> clusterCentroidArrayMap;
	public Map<Integer, Integer> indexToClusterMap = new TreeMap<Integer, Integer>();
	private Map<Integer, Map<String, Double>> attributeToClusterMap;
	private Map<String, Integer> labelIndexMap;

	private SortedSet<String> attributeNames;
	private SortedSet<Integer> clusters;
	private Set<Integer> attributeSet;

	private ArrayList<ArrayList<Double>> list;
	private double actualThreshold;
	private Clustering<Model> c;
	private Relation<NumberVector> rel;
	private DBIDRange ids;
	protected ModifiableDBIDs noise;

	private boolean noiseToCluster;

	public void buildIt(double[][] data, String[] labels) {
		setLabels(labels);
		setData(data);

		labelIndexMap = new HashMap<String, Integer>();
		for (int i = 0; i < labels.length; i++) {
			labelIndexMap.put(labels[i], i);
		}

		//		DatabaseConnection dbc = new ArrayAdapterDatabaseConnection(data, labels);
		DatabaseConnection dbc = new ArrayAdapterDatabaseConnection(data);
		Database db = new StaticArrayDatabase(dbc, null);
		db.initialize();

		// dbscan algorithm setup
		ListParameterization params = new ListParameterization();
		params.addParameter(DBSCAN.Parameterizer.EPSILON_ID, getEpsilon());
		params.addParameter(DBSCAN.Parameterizer.MINPTS_ID, getMinPoints());
		DBSCAN<DoubleVector> dbscan = ClassGenericsUtil.parameterizeOrAbort(DBSCAN.class, params);
		// Run DBSCAN on database
		Clustering<Model> c = dbscan.run(db);
		setClustering(c);

		// Relation containing the number vectors;
		Relation<NumberVector> rel = db.getRelation(TypeUtil.NUMBER_VECTOR_FIELD);
		setRelation(rel);
		// We know that the ids must be a continuous range:
		DBIDRange ids = (DBIDRange) rel.getDBIDs();
		setRange(ids);
		noise = DBIDUtil.newArray();

		clusterCentroidMap = new TreeMap<Integer, NumberVector>();
		clusterCentroidArrayMap = new TreeMap<Integer, double[]>();
		attributeToClusterMap = new TreeMap<Integer, Map<String, Double>>();
		clusters = new TreeSet<Integer>();
		int x = 0;
		for (Cluster<Model> clu : c.getAllClusters()) {
			if (clu.isNoise()) {
				noise.addDBIDs(clu.getIDs());
				isNoise = true;
				noiseCluster = x;
				System.out.println("noise cluster = " + noiseCluster);
			}

			double[] sum = new double[labels.length];
			int counter = 0;
			if (!clu.isNoise()) {
				if (!attributeToClusterMap.containsKey(x)) {
					attributeToClusterMap.put(x, new TreeMap<String, Double>());
				}
				clusters.add(x);

				Map<String, Double> m = attributeToClusterMap.get(x);
				for (DBIDIter it = clu.getIDs().iter(); it.valid(); it.advance()) {
					NumberVector v = rel.get(it);
					for (int y = 0; y < labels.length; y++) {
						sum[y] = sum[y] + v.doubleValue(y);

						if (!m.containsKey(labels[y])) {
							m.put(labels[y], v.doubleValue(y));
						} else {
							m.put(labels[y], m.get(labels[y]) + v.doubleValue(y));
						}
					}
					counter++;
					this.indexToClusterMap.put(ids.getOffset(it), x);
				}
				for (int i = 0; i < sum.length; i++) {
					sum[i] = sum[i]/counter;
				}
				clusterCentroidArrayMap.put(x, sum);
				NumberVector centroid = new DoubleVector.Factory().newNumberVector(sum);
				clusterCentroidMap.put(x, centroid);
			}
			x++;
		}

		System.out.println("cluster set = " + clusters);
		this.setNumberOfClusters(getClusters().size());	
		
		if (getWithNoiseToCluster() == true) {
			assignNoiseToCluster();
		}

		attributeNames = new TreeSet<String>();
		for (String name : labels) {
			attributeNames.add(name);
		}

		list = new ArrayList<ArrayList<Double>>();
		for (int cluster : clusters) {
			ArrayList<Double> centerValues = new ArrayList<Double>();
			NumberVector center = clusterCentroidMap.get(cluster);
			for (int i = 0; i < labels.length; i++) {
				centerValues.add(center.doubleValue(i));
			}
			list.add(centerValues);
		}

		// Create the matrix (simple way)
		dataMatrix = new double[list.size()][];
		for (int k = 0; k < list.size(); k++) {
			ArrayList<Double> row = list.get(k);

			double[] copy = new double[row.size()];			
			for (int j = 0; j < row.size(); j++) {
				copy[j] = row.get(j);
			}
			dataMatrix[k] = copy;
		}

		oldDataMatrix = cloneArray(dataMatrix);
		dataMatrix = transposeMatrix(dataMatrix);

		for (int i = 0; i < dataMatrix.length; i++) {
			for (int j = 0; j < dataMatrix[i].length; j++) {
				if (dataMatrix[i][j] > maxValue) {
					maxValue = dataMatrix[i][j];
				}
				if (dataMatrix[i][j] < minValue) {
					minValue = dataMatrix[i][j];
				}
			}
		}

	}

	public void runFiltered(double[][] data) {
		double[] columnMax = columnMax(data);
		attributeSet = new HashSet<Integer>();
		for (int i = 0; i < columnMax.length; i++) {
			if (columnMax[i] < getActualThresholdValue()) {
				attributeSet.add(i);
			}
		}

		List<Integer> attIndexArray = new ArrayList<Integer>(attributeSet);
		Collections.sort(attIndexArray, Collections.reverseOrder());

		List<String> labelsList = new ArrayList<>(Arrays.asList(labels));
		for (int i : attIndexArray) {
			labelsList.remove(i);
		}

		String[] filteredLabels = labelsList.toArray(new String[labelsList.size()]);

		attributeNames = new TreeSet<String>();
		for (String name : filteredLabels) {
			attributeNames.add(name);
		}

		Map<Integer, NumberVector> newClusterCentroidMap = new TreeMap<Integer, NumberVector>();
		for (int cluster : clusterCentroidMap.keySet()) {
			NumberVector centroid = clusterCentroidMap.get(cluster);
			double[] newCenter = new double[filteredLabels.length];
			for (int i = 0; i < filteredLabels.length; i++) {
				newCenter[i] = centroid.doubleValue(labelIndexMap.get(filteredLabels[i]));
			}
			NumberVector newCentroid = new DoubleVector.Factory().newNumberVector(newCenter);
			newClusterCentroidMap.put(cluster, newCentroid);
		}

		list = new ArrayList<ArrayList<Double>>();
		for (int cluster : clusters) {
			ArrayList<Double> centerValues = new ArrayList<Double>();
			NumberVector center = newClusterCentroidMap.get(cluster);
			for (int i = 0; i < filteredLabels.length; i++) {
				centerValues.add(center.doubleValue(i));
			}
			list.add(centerValues);
		}

		dataMatrix = new double[list.size()][];
		for (int k = 0; k < list.size(); k++) {
			ArrayList<Double> row = list.get(k);

			double[] copy = new double[row.size()];			
			for (int j = 0; j < row.size(); j++) {
				copy[j] = row.get(j);
			}
			dataMatrix[k] = copy;
		}

		dataMatrix = transposeMatrix(dataMatrix);
	}

	private static double[] columnMax(double[][] array) {
		double[] result = array[0];
		for (int row = 1; row < array.length; row++) {
			for (int column = 0; column < array[0].length; column++) {
				if (array[row][column] > result[column]) {
					result[column] = array[row][column];
				}
			}
		}
		return result;
	}

	public static double[][] cloneArray(double[][] src) {
		int length = src.length;
		double[][] target = new double[length][src[0].length];
		for (int i = 0; i < length; i++) {
			System.arraycopy(src[i], 0, target[i], 0, src[i].length);
		}
		return target;
	}

	private void assignNoiseToCluster() {
		for (DBIDMIter iter = noise.iter(); iter.valid(); iter.advance()) {
			double dist = Double.MAX_VALUE;
			int clust = 0;
			NumberVector noiseV = rel.get(iter);
			double[] noise = new double[labels.length];
			for (int i = 0; i < labels.length; i++) {
				noise[i] = noiseV.doubleValue(i);
			}

			for (int cluster : clusterCentroidArrayMap.keySet()) {
				double[] centroid = clusterCentroidArrayMap.get(cluster);
				PearsonsCorrelation pearsonDist = new PearsonsCorrelation();
				double distance = pearsonDist.correlation(noise, centroid);
				if (distance < dist) {
					dist = distance;
					clust = cluster;
				}
			}
			this.indexToClusterMap.put(ids.getOffset(iter), clust);
		}

	}

	public static double[][] transposeMatrix(double[][] matrix) {
		int m = matrix.length;
		int n = matrix[0].length;

		double[][] transposedMatrix = new double[n][m];

		for(int x = 0; x < n; x++) {
			for(int y = 0; y < m; y++) {
				transposedMatrix[x][y] = matrix[y][x];
			}
		}
		return transposedMatrix;
	}

	public void setMinPoints(int minPoints) {
		this.minPoints = minPoints;
	}

	private Object getMinPoints() {
		return minPoints;
	}

	public void setEpsilon(double epsilon) {
		this.epsilon = epsilon;
	}

	private Object getEpsilon() {
		return epsilon;
	}

	private void setData(double[][] data) {
		this.data = data;		
	}

	private void setLabels(String[] labels) {
		this.labels = labels;
	}

	public void setNumberOfClusters (int numberOfClusters) {
		this.numberOfClusters = numberOfClusters;
	}

	public int getNumberOfClusters() {
		return numberOfClusters;
	}

	public double getMinValue() {
		return minValue;
	}

	public double getMaxValue() {
		return maxValue;
	}

	public SortedSet<Integer> getClusters() {
		return clusters;
	}

	public SortedSet<String> getAttributeNames() {
		return attributeNames;
	}

	public double[][] getMatrix() {
		return dataMatrix;
	}

	public double[][] getOldMatrix() {
		return oldDataMatrix;
	}

	public void setActualThresholdValue(double value) {
		this.actualThreshold = value;
	}

	public double getActualThresholdValue() {
		return actualThreshold;
	}

	public Map<Integer, Map<String, Double>> getClusterToAttributeValueMap() {
		return attributeToClusterMap;
	}

	private void setClustering(Clustering<Model> c) {
		this.c = c;
	}

	private Clustering<Model> getClustering() {
		return c;
	}

	private void setRelation(Relation<NumberVector> rel) {
		this.rel = rel;
	}

	private Relation<NumberVector> getRelation() {
		return rel;
	}

	private void setRange(DBIDRange ids) {
		this.ids = ids;
	}

	private DBIDRange getRange() {
		return ids;
	}	

	public Map<Integer, Integer> getClusterAssignments() {
		return indexToClusterMap;
	}

	public int getNoiseCluster() {
		return noiseCluster;
	}

	public void setWithNoiseToCluster(boolean checked) {
		this.noiseToCluster = checked;		
	}

	public Boolean getWithNoiseToCluster() {
		return noiseToCluster;
	}

}
