package org.processmining.timebasedabstraction.algorithms;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import weka.clusterers.SimpleKMeans;
import weka.core.Attribute;
import weka.core.AttributeStats;
import weka.core.Instance;
import weka.core.Instances;
import weka.experiment.Stats;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.Remove;

public class KMeans {

	private SimpleKMeans kmeans = new SimpleKMeans();
	private Set<Integer> attributeSet;
	private Instances centroids;
	private int[] assignments;
	private int numberOfClusters;

	private ArrayList<Double> avgSums;
	private ArrayList<ArrayList<Double>> list;
	private Map<Integer, Map<String, Double>> attributeToClusterMap;
	private SortedSet<String> attributeNames;
	private SortedSet<Integer> clusters;
	private double actualThreshold;

	private double[][] dataMatrix;
	private double maxValue = 0; 
	private double minValue = Double.MAX_VALUE;

	public void buildIt(Instances data) throws Exception {	
		kmeans.setPreserveInstancesOrder(true);
		data.randomize(new Random());
		kmeans.setNumClusters(getNumberOfClusters());
		kmeans.buildClusterer(data);

		centroids = kmeans.getClusterCentroids();
		System.out.println("centroid: " + centroids);
		for (int i = 0; i < centroids.numAttributes(); i++) {
			AttributeStats attStats = centroids.attributeStats(i);
			Stats stats  = attStats.numericStats;
			double sum = stats.sum;
			double avgSum = sum / centroids.numInstances();
			if (avgSum > maxValue) {
				maxValue = avgSum;
			}
			if (avgSum < minValue) {
				minValue = avgSum;
			}
		}

		assignments = kmeans.getAssignments();
		attributeToClusterMap = new TreeMap<Integer, Map<String, Double>>();
		for (int i = 0; i < centroids.size(); i++) {
			if (!attributeToClusterMap.containsKey(i)) {
				attributeToClusterMap.put(i, new TreeMap<String, Double>());
			}
		}

		clusters = new TreeSet<Integer>();
		for (int i = 0; i < centroids.size(); i++) {
			clusters.add(i);
		}

		list = new ArrayList<ArrayList<Double>>();
		for (int i = 0; i < centroids.size(); i++) {
			Map<String, Double> m = attributeToClusterMap.get(i);
			Instance instance = centroids.get(i);
			ArrayList<Double> values = new ArrayList<Double>();
			for (int j = 0; j < centroids.numAttributes(); j++) {
				Attribute attribute = instance.attribute(j);
				double instanceValue = instance.value(attribute);
				values.add(instanceValue);
				if (!m.containsKey(attribute.name())) {
					m.put(attribute.name(), instanceValue);
				} else {
					m.put(attribute.name(), m.get(attribute.name()) + instanceValue);
				}
			}
			list.add(values);
		}
		
		attributeNames = new TreeSet<String>();
		for (Integer key : attributeToClusterMap.keySet()) {
			for (String attNames : attributeToClusterMap.get(key).keySet()) {
				attributeNames.add(attNames);
			}
		}

		// Create the matrix (simple way)
		dataMatrix = new double[list.size()][];
		for (int i = 0; i < list.size(); i++) {
			ArrayList<Double> row = list.get(i);
			double[] copy = new double[row.size()];
			for (int j = 0; j < row.size(); j++) {
				copy[j] = row.get(j);
			}
			dataMatrix[i] = copy;
		}
		
		dataMatrix = transposeMatrix(dataMatrix);
	}

	public void runFiltered(Instances data) throws Exception {
		centroids = kmeans.getClusterCentroids();
		attributeSet = new HashSet<Integer>();
		for (int i = 0; i < centroids.numAttributes(); i++) {
			Attribute att = centroids.attribute(i);
			AttributeStats attStats = centroids.attributeStats(i);
			Stats stats  = attStats.numericStats;
			double sum = stats.sum;
			double avgSum = sum / centroids.numInstances();

			if (avgSum < getActualThresholdValue()) {
				attributeSet.add(att.index());
			}
		}

		Remove filter = new Remove();
		int[] attributeIndex = new int[attributeSet.size()];

		for (int i = 0; i < attributeIndex.length; i++) {
			for (int value : attributeSet) {
				attributeIndex[i++] = value;
			}
		}

		filter.setAttributeIndicesArray(attributeIndex);
		filter.setInputFormat(centroids);
		Instances filteredCentroids = Filter.useFilter(centroids, filter);

		assignments = kmeans.getAssignments();
		attributeToClusterMap = new TreeMap<Integer, Map<String, Double>>();
		for (int i = 0; i < centroids.size(); i++) {
			if (!attributeToClusterMap.containsKey(i)) {
				attributeToClusterMap.put(i, new TreeMap<String, Double>());
			}
		}
		
		for (Integer key : attributeToClusterMap.keySet()) {
			Map<String, Double> m = attributeToClusterMap.get(key);
			for (int i = 0; i < attributeToClusterMap.size(); i++) {
				Instance instance = filteredCentroids.get(i);
				for (int j = 0; j < filteredCentroids.numAttributes(); j++) {
					Attribute attribute = instance.attribute(j);
					double instanceValue = instance.value(attribute);
					if (!m.containsKey(attribute.name())) {
						m.put(attribute.name(), instanceValue);
					} else {
						m.put(attribute.name(), m.get(attribute.name()) + instanceValue);
					}					
				}
			}
		}
		
		attributeNames = new TreeSet<String>();
		for (Integer key : attributeToClusterMap.keySet()) {
			for (String attNames : attributeToClusterMap.get(key).keySet()) {
				attributeNames.add(attNames);
			}
		}

		clusters = new TreeSet<Integer>();
		for (int clusterNumb : assignments) { // kmeans.getAssignments()
			clusters.add(clusterNumb);
		}

		list = new ArrayList<ArrayList<Double>>();
		for (int i = 0; i < filteredCentroids.size(); i++) {
			Instance instance = filteredCentroids.get(i);
			ArrayList<Double> values = new ArrayList<Double>();
			for (int j = 0; j < filteredCentroids.numAttributes(); j++) {
				Attribute attribute = instance.attribute(j);
				double instanceValue = instance.value(attribute);
				values.add(instanceValue);
			}
			list.add(values);
		}

		// Create the matrix (simple way)
		dataMatrix = new double[list.size()][];
		for (int i = 0; i < list.size(); i++) {
			ArrayList<Double> row = list.get(i);
			double[] copy = new double[row.size()];
			for (int j = 0; j < row.size(); j++) {
				copy[j] = row.get(j);
			}
			dataMatrix[i] = copy;
		}
		
		dataMatrix = transposeMatrix(dataMatrix);
	}
	
	public static double[][] transposeMatrix(double[][] matrix) {
		int m = matrix.length;
		int n = matrix[0].length;

		double[][] transposedMatrix = new double[n][m];

		for(int x = 0; x < n; x++) {
			for(int y = 0; y < m; y++) {
				transposedMatrix[x][y] = matrix[y][x];
			}
		}
		return transposedMatrix;
	}

	public SortedSet<Integer> getClusters() { 
		return clusters;
	}

	public SortedSet<String> getAttributeNames() { 
		return attributeNames;
	}

	public double[][] getMatrix() {
		return dataMatrix;
	}

	public void setNumberOfClusters(int numberOfClusters) {
		this.numberOfClusters = numberOfClusters;
	}

	public int getNumberOfClusters() {
		return numberOfClusters;
	}

	public void setActualThresholdValue(double value) {
		this.actualThreshold = value;
	}

	public double getActualThresholdValue() {
		return actualThreshold;
	}

	public double getMinValue() {
		return minValue;
	}

	public double getMaxValue() {
		return maxValue;
	}

	public int[] getClusterAssignments() {
		return assignments;
	}

	public Map<Integer, Map<String, Double>> getClustertoAttributeValueMap() {
		return attributeToClusterMap;
	}


}
