package org.processmining.timebasedabstraction.export;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryNaiveImpl;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.timebasedabstraction.models.TimeAndCountModel;
import org.processmining.timebasedabstraction.plugins.CreateAbstractedLog;

public class Export15Min extends CreateAbstractedLog {
	/**
	 * NextEvent > 15 Min.
	 */
	public XLog mineAbstractLog15Min(XLog log, TimeAndCountModel model, boolean withNumbers) {

		XFactory factory = new XFactoryNaiveImpl();
		XConceptExtension xce = XConceptExtension.instance();
		XTimeExtension xte = XTimeExtension.instance();
		XLifecycleExtension xlce = XLifecycleExtension.instance();

		Map<Integer, Integer> indexToClusterMap = model.getIndextToClusterMap();
		Map<Integer, String> clusterNameMap = model.getClusterNameMap();
		ArrayList<XEvent> eventlist;
		XEvent a;
		XEvent b;
		XTrace newTrace;

		double currentEventTime;
		double nextEventTime;
		double timeDiff;
		int counter = 0;
		XEvent nextEvent;

		XLog newLog = factory.createLog();
		newLog.setAttributes(log.getAttributes());

		for (XTrace trace : log) {
			Object[] traceArray = trace.toArray();
			eventlist = new ArrayList<XEvent>();
			newTrace = factory.createTrace();
			newTrace.setAttributes(trace.getAttributes());
			Set<String> namesSet = new HashSet<String>();
			for (int i = 0; i < traceArray.length - 1; i++) {
				boolean inBounds = (((i + 1) >= 0) && ((i + 1) < traceArray.length));

				XEvent currentEvent = (XEvent) traceArray[i];
				if (inBounds) {
					nextEvent = (XEvent) traceArray[i + 1];
				} else {
					nextEvent = null;
				}

				currentEventTime = xte.extractTimestamp(currentEvent).getTime();
				if (nextEvent == null) {
					nextEventTime = 0.0d;
				} else {
					nextEventTime = xte.extractTimestamp(nextEvent).getTime();
				}
				timeDiff = nextEventTime - currentEventTime;

				if (timeDiff >= 0 && timeDiff < TimeUnit.MINUTES.toMillis(15)) {
					eventlist.add(currentEvent);
				} else {
					eventlist.add(currentEvent);	    			
					// TODO do something

					if (indexToClusterMap.get(counter) != null) {
						a = factory.createEvent();	    			
						a.setAttributes(eventlist.get(0).getAttributes());
						xce.assignName(a, clusterNameMap.get(indexToClusterMap.get(counter)));	    			
						xte.assignTimestamp(a, xte.extractTimestamp(eventlist.get(0)));
						xlce.assignStandardTransition(a, XLifecycleExtension.StandardModel.START);

						b = factory.createEvent();	    			
						b.setAttributes(eventlist.get(eventlist.size() - 1).getAttributes());
						xce.assignName(b, clusterNameMap.get(indexToClusterMap.get(counter)));
						xte.assignTimestamp(b, xte.extractTimestamp(eventlist.get(eventlist.size() - 1)));
						xlce.assignStandardTransition(b, XLifecycleExtension.StandardModel.COMPLETE);

						if (withNumbers == true) {
							if (namesSet.isEmpty()) {
								namesSet.add(xce.extractName(a));
								newTrace.add(a);
								newTrace.add(b);
							} else {
								if (namesSet.contains(xce.extractName(a))) {
									int key = (int) getKeyFromValue(clusterNameMap, xce.extractName(a));
									String name = clusterNameMap.get(indexToClusterMap.get(counter));
									if (stringContainsNumber(name)) {
										name = incrementOne(name);
										clusterNameMap.put(key, name);
									} else {
										clusterNameMap.put(key, clusterNameMap.get(indexToClusterMap.get(counter)) + 1);
									}
									xce.assignName(a, clusterNameMap.get(key));
									xce.assignName(b, clusterNameMap.get(key));

									namesSet.add(xce.extractName(a));

									newTrace.add(a);
									newTrace.add(b);
								} else {
									newTrace.add(a);
									newTrace.add(b);
									namesSet.add(xce.extractName(a));
								}
							}
						} else {
							newTrace.add(a);
							newTrace.add(b);
						}
					} else {
						counter++;
						eventlist = new ArrayList<XEvent>();
						continue;
					}

					counter++;
					eventlist = new ArrayList<XEvent>();
				}	 

			}
			newLog.add(newTrace);
		}

		System.out.println("Abstract Log Generated Succesfully");
		return newLog;
	}
}
