package org.processmining.timebasedabstraction.export;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.time.DateUtils;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryNaiveImpl;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.timebasedabstraction.models.TimeAndCountModel;
import org.processmining.timebasedabstraction.parameters.Parameters;
import org.processmining.timebasedabstraction.plugins.CreateAbstractedLog;

public class ExportMonths extends CreateAbstractedLog {
	/**
	 * Months
	 */
	public XLog mineAbstractLogMonths(XLog log, TimeAndCountModel model, boolean withNumbers) {
		XFactory factory = new XFactoryNaiveImpl();
		XConceptExtension xce = XConceptExtension.instance();
		XTimeExtension xte = XTimeExtension.instance();
		XLifecycleExtension xlce = XLifecycleExtension.instance();

		Map<Integer, Integer> indexToClusterMap = model.getIndextToClusterMap();
		Map<Integer, String> clusterNameMap = model.getClusterNameMap();
		ArrayList<XEvent> eventlist;
		XEvent a;
		XEvent b;
		XTrace newTrace;

		Parameters parameters = model.getParameterValues();
		int value = parameters.getTextFieldValue();

		double currentEventTime;
		Date sessionStartDate;
		double sessionStartTime;
		Date sessionEndDate;
		double sessionEndTime = 0.0d;
		double nextEventTime;
		double timeDiff;
		XEvent nextEvent;
		XEvent startSessionEvent;
		int counter = 0;

		XLog newLog = factory.createLog();
		newLog.setAttributes(log.getAttributes());

		for (XTrace trace : log) {
			Object[] traceArray = trace.toArray();
			eventlist = new ArrayList<XEvent>();
			newTrace = factory.createTrace();
			newTrace.setAttributes(trace.getAttributes());
			Set<String> namesSet = new HashSet<String>();
			boolean flag = true;
			for (int i = 0; i < traceArray.length; i++) {
				boolean inBounds = ((i + 1) >= 0) && ((i + 1) < traceArray.length);

				XEvent currentEvent = (XEvent) traceArray[i];
				currentEventTime = XTimeExtension.instance().extractTimestamp(currentEvent).getTime();

				if (inBounds) {
					nextEvent = (XEvent) traceArray[i + 1];
				} else {
					nextEvent = null;
				}

				if (nextEvent == null) {
					nextEventTime = 0.0d;
				} else {
					nextEventTime = XTimeExtension.instance().extractTimestamp(nextEvent).getTime();
				}
				timeDiff = nextEventTime - currentEventTime;

				if (flag) {
					startSessionEvent = (XEvent) traceArray[0];
					sessionStartDate = XTimeExtension.instance().extractTimestamp(startSessionEvent);
					sessionStartTime = sessionStartDate.getTime();
					sessionEndDate = DateUtils.addMonths(sessionStartDate, value);
					sessionEndTime = sessionEndDate.getTime();
					flag = false;
				}

				if (nextEventTime <= sessionEndTime && timeDiff >= 0) {
					eventlist.add(currentEvent);
				} else {
					eventlist.add(currentEvent);

					if (indexToClusterMap.get(counter) != null) {
						a = factory.createEvent();        
						a.setAttributes(eventlist.get(0).getAttributes());
						xce.assignName(a, clusterNameMap.get(indexToClusterMap.get(counter)));        
						xte.assignTimestamp(a, xte.extractTimestamp(eventlist.get(0)));
						xlce.assignStandardTransition(a, XLifecycleExtension.StandardModel.START);

						b = factory.createEvent();        
						b.setAttributes(eventlist.get(eventlist.size() - 1).getAttributes());
						xce.assignName(b, clusterNameMap.get(indexToClusterMap.get(counter)));
						xte.assignTimestamp(b, xte.extractTimestamp(eventlist.get(eventlist.size() - 1)));
						xlce.assignStandardTransition(b, XLifecycleExtension.StandardModel.COMPLETE);

						if (withNumbers == true) {
							if (namesSet.isEmpty()) {
								namesSet.add(xce.extractName(a));
								newTrace.add(a);
								newTrace.add(b);
							} else {
								if (namesSet.contains(xce.extractName(a))) {
									int key = (int) getKeyFromValue(clusterNameMap, xce.extractName(a));
									String name = clusterNameMap.get(indexToClusterMap.get(counter));
									if (stringContainsNumber(name)) {
										name = incrementOne(name);
										clusterNameMap.put(key, name);
									} else {
										clusterNameMap.put(key, clusterNameMap.get(indexToClusterMap.get(counter)) + 1);
									}
									xce.assignName(a, clusterNameMap.get(key));
									xce.assignName(b, clusterNameMap.get(key));

									namesSet.add(xce.extractName(a));

									newTrace.add(a);
									newTrace.add(b);
								} else {
									newTrace.add(a);
									newTrace.add(b);
									namesSet.add(xce.extractName(a));
								}
							}
						} else {
							newTrace.add(a);
							newTrace.add(b);
						}
					} else {
						counter++;
						eventlist = new ArrayList<XEvent>();
						continue;
					}

					counter++;
					eventlist = new ArrayList<XEvent>();

					if (nextEvent != null) {
						startSessionEvent = nextEvent;
						sessionStartDate = XTimeExtension.instance().extractTimestamp(startSessionEvent);
						sessionStartTime = sessionStartDate.getTime();
						sessionEndDate = DateUtils.addMonths(sessionStartDate, value);
						sessionEndTime = sessionEndDate.getTime(); 
					} else {
						break;
					}
				}
			}
			if (newTrace.isEmpty()) {
				continue;
			} else {
				newLog.add(newTrace);
			}
		}

		System.out.println("Abstract Log Generated Succesfully");
		return newLog;
	}


}
