package org.processmining.timebasedabstraction.mineTime;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.timebasedabstraction.models.TimeAndCountModel;
import org.processmining.timebasedabstraction.parameters.Parameters;

public class MineHours {
	public TimeAndCountModel mineTime(PluginContext context, XLog log, Parameters parameters) {
		context.getProgress().setMaximum(log.size());
		  XLogInfo info = XLogInfoFactory.createLogInfo(log, parameters.getClassifier());
		  int value = parameters.getTextFieldValue();
//		  System.out.println(value);
		  String item = parameters.getComboBoxItem();
//		  System.out.println(item);
		  
		  TimeAndCountModel model = new TimeAndCountModel();
		     model.setEventLog(log);
		     model.setEventClasses(info);
		     model.setKeys();
		     Map<XEventClass, Double> eventMap;
		     double currentEventTime;
		     double nextEventTime;
		     double timeDiff;
		     XEvent nextEvent;
		     
		     mineLogForTimeHours(model, value);
		     
		     for (XTrace trace : log) {
		         eventMap = new LinkedHashMap<XEventClass, Double>();
		         model.setEventMap(eventMap);
		         Object[] traceArray = trace.toArray();
		         for (int i = 0; i < traceArray.length; i++) {
		             boolean inBounds = ((i + 1) >= 0) && ((i + 1) < traceArray.length);
		             
		             XEvent currentEvent = (XEvent) traceArray[i];
		             currentEventTime = XTimeExtension.instance().extractTimestamp(currentEvent).getTime();
		             
		             if (inBounds) {
		                 nextEvent = (XEvent) traceArray[i + 1];
		             } else {
		                 nextEvent = null;
		             }
		             
		             if (nextEvent == null) {
		                 nextEventTime = 0.0d;
		             } else {
		                 nextEventTime = XTimeExtension.instance().extractTimestamp(nextEvent).getTime();
		             }
		             timeDiff = nextEventTime - currentEventTime;
		            
		             
		             if (timeDiff >= 0 && timeDiff < TimeUnit.HOURS.toMillis(value)) {
		                 model.addToEventMap(currentEvent, timeDiff);
		             } else {
		                 double avg = (model.getTimeEventMap().get(model.getEventClasses().getClassOf(currentEvent))) / (model.getCounterMap().get(model.getEventClasses().getClassOf(currentEvent)));
		                 model.addToEventMap(currentEvent, avg);     
		                 
		                 if (!model.getEventMap().isEmpty()) {
		                     model.addToEventMapList(eventMap);
		                 }
		                 
		                 eventMap = new LinkedHashMap<XEventClass, Double>();
		                 model.setEventMap(eventMap);
		             }
		         }
		         context.getProgress().inc();
		     }
		     
		     // Normalize the data
		     model.normalizeData();
		     // Create the Weka dataset
		     model.createDataSet();
		     // Create ELKi dataset
		     model.createElkiDataset();
		     
		     return model;
	}

	private void mineLogForTimeHours(TimeAndCountModel model, int value) {
		XLog log = model.getEventLog();
	     Map<XEventClass, Double> timeEventMap = new LinkedHashMap<XEventClass, Double>();
	     Map<XEventClass, Integer> counterMap = new LinkedHashMap<XEventClass, Integer>();
	     
	     model.setTimeEventMap(timeEventMap);
	     model.setCounterMap(counterMap);
	     
	     double currentEventTime;
	     double nextEventTime;
	     double timeDiff;
	     
	     XEvent nextEvent;
	     
	     for (XTrace trace : log) {
	         Object[] traceArray = trace.toArray();
	         for (int i = 0; i < traceArray.length; i++) {
	             boolean inBounds = ((i + 1) >= 0) && ((i + 1) < traceArray.length);
	             
	             XEvent currentEvent = (XEvent) traceArray[i];
	             if (inBounds) {
	                 nextEvent = (XEvent) traceArray[i + 1];
	             } else {
	                 nextEvent = null;
	             }
	             
	             currentEventTime = XTimeExtension.instance().extractTimestamp(currentEvent).getTime();
	             if (nextEvent == null) {
	                 nextEventTime = 0.0d;
	             } else {
	                 nextEventTime = XTimeExtension.instance().extractTimestamp(nextEvent).getTime();
	             }
	             timeDiff = nextEventTime - currentEventTime;
	             
	             if (timeDiff >= 0 && timeDiff < TimeUnit.HOURS.toMillis(value)) {
	                 model.totalTimeEventMap(currentEvent, timeDiff);
	                 model.addToCounterMap(currentEvent, 1);
	             } else {
	                 model.totalTimeEventMap(currentEvent, 0.0d);
	                 model.addToCounterMap(currentEvent, 1);
	             }
	         }
	     }
	}
	
	
}
