package org.processmining.timebasedabstraction.models;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.processmining.framework.util.HTMLToString;
import org.processmining.timebasedabstraction.parameters.Parameters;

import weka.core.Attribute;
import weka.core.DenseInstance;
import weka.core.Instances;

public class TimeAndCountModel implements HTMLToString {
	private Map<Integer, String> clusterNameMap;
	private Map<Integer, Integer> indexToClusterMap;	

	private XLog log;
	private XEventClasses eventClasses;

	private List<Map<XEventClass, Double>> eventMapList = new ArrayList<Map<XEventClass, Double>>();
	private Map<XEventClass, Double> eventMap;
	private Map<XEventClass, Double> timeEventMap;
	private Map<XEventClass, Integer> counterMap;
	private SortedSet<XEventClass> keys;

	private Instances data;
	private double[][] elkiData;
	Parameters parameter;
	private String[] labels;

	public void setEventLog(XLog log) {
		this.log = log;
	}

	public XLog getEventLog() {
		return log;
	}

	public void setEventClasses(XLogInfo info) {
		this.eventClasses = info.getEventClasses();
	}

	public XEventClasses getEventClasses() {
		return eventClasses;
	}

	public void setKeys() {
		this.keys = new TreeSet<XEventClass>(eventClasses.getClasses());
	}

	public void setEventMap(Map<XEventClass, Double> eventMap) {
		this.eventMap = eventMap;
	}

	public Map<XEventClass, Double> getEventMap() {
		return eventMap;
	}
	
	public void setTimeEventMap(Map<XEventClass, Double> timeEventMap) {
		this.timeEventMap = timeEventMap;
	}
	
	public Map<XEventClass, Double> getTimeEventMap() {
		return timeEventMap;
	}
	
	public void setCounterMap(Map<XEventClass, Integer> counterMap) {
		this.counterMap = counterMap;
	}
	
	public Map<XEventClass, Integer> getCounterMap() {
		return counterMap;
	}

	public void addToEventMap(XEvent event, double count) {
		XEventClass eventClass = eventClasses.getClassOf(event);
		if (!eventMap.containsKey(eventClass)) {
			eventMap.put(eventClass, count);
		} else {
			eventMap.put(eventClass, eventMap.get(eventClass) + count);
		}
	}
	
	public void totalTimeEventMap(XEvent event, double timediff) {
		XEventClass eventClass = eventClasses.getClassOf(event);
		if (!timeEventMap.containsKey(eventClass)) {
			timeEventMap.put(eventClass, timediff);
		} else {
			timeEventMap.put(eventClass, timeEventMap.get(eventClass) + timediff);
		}
	}
	
	public void addToCounterMap(XEvent event, int count) {
		XEventClass eventClass = eventClasses.getClassOf(event);
		if (!counterMap.containsKey(eventClass)) {
			counterMap.put(eventClass, count);
		} else {
			counterMap.put(eventClass, counterMap.get(eventClass) + count);
		}
	}

	public void addToEventMapList(Map<XEventClass, Double> eventMap) {
		eventMapList.add(eventMap);
	}

	public List<Map<XEventClass, Double>> getEventMapList() {
		return eventMapList;
	}

	public Instances createDataSet() {
		this.data = new Instances("UWV", createAttributes(), 0);

		for (Map<XEventClass, Double> e : this.eventMapList) {
			double[] instanceArray = new double[this.keys.size()];
			
			int i=0;
			for (XEventClass key : this.keys) {
				instanceArray[i++]=(e.get(key) == null ? 0 : e.get(key));
			}

//			this.data.add(new SparseInstance(1, instanceArray));
			this.data.add(new DenseInstance(1, instanceArray));
		}
		
		System.out.println("Weka data size: " + data.size());
		return data;
	}
	
	public double[][] createElkiDataset() {
		setLabels();
		this.elkiData = new double[this.eventMapList.size()][];
		int i = 0;
		for (Map<XEventClass, Double> e : this.eventMapList) {
			
			double[] dataArray = new double[this.keys.size()];
			
			int j = 0;
			for (XEventClass key : this.keys) {
				dataArray[j++] = (e.get(key) == null ? 0 : e.get(key));
				
			}
			elkiData[i++] = dataArray;
						
		}
		
		System.out.println("elkiData size: " + elkiData.length);
		return elkiData;
	}

	public Instances getDataSet() {
		return data;
	}
	
	public double[][] getElkiDataSet() {
		return elkiData;
	}

	private ArrayList<Attribute> createAttributes() {
		ArrayList<Attribute> attributes = new ArrayList<Attribute>();
		for (XEventClass key : this.keys) {
			attributes.add(new Attribute(key.toString()));
		}

		return attributes;
	}
	
	private void setLabels() {
		ArrayList<String> labels = new ArrayList<String>();
		for (XEventClass key : this.keys) {
			labels.add(key.toString());
		}
		
		String[] labelsArray = labels.toArray(new String[labels.size()]);
		this.labels = labelsArray;
	}
	
	public String[] getLabels() {
		return labels;
	}

	public void normalizeData() {		
		for (Map<XEventClass, Double> map : this.eventMapList) {
			double sum = 0;
			
			for(double val : map.values()) {
				sum+=val;
			}
			
			for (XEventClass key : map.keySet()) {
				double normalizedValue = map.get(key) / sum;
				if (Double.isNaN(normalizedValue)) {
					map.put(key, 0.0d);
				} else {
					map.put(key, normalizedValue);
				}
			}
		}
	}

	public void setClusterNameMap(Map<Integer, String> clusterNameMap) {
		this.clusterNameMap = clusterNameMap;
	}

	public Map<Integer, String> getClusterNameMap() {
		return clusterNameMap;
	}

	public void setIndexToClusterMap(Map<Integer, Integer> indexToClusterMap) {
		this.indexToClusterMap = indexToClusterMap;
	}

	public Map<Integer, Integer> getIndextToClusterMap() {
		return indexToClusterMap;
	}

	
	public void setParameterValues(Parameters parameters) {
		this.parameter = parameters;
	}
	
	public Parameters getParameterValues() {
		return parameter;
	}
	
	public String toHTMLString(boolean includeHTMLTags) {
		StringBuffer buffer = new StringBuffer();
		if (includeHTMLTags) {
			buffer.append("<html>");
		}
		buffer.append("<table>");
		for (Map<XEventClass, Double> map : getEventMapList()) {
			buffer.append("<tr>");
			buffer.append("<td>" + map + "</td>");
			buffer.append("</tr>");
		}
		buffer.append("</table>");
		if (includeHTMLTags) {
			buffer.append("</html>");
		}
		return buffer.toString();
	}


}
