package org.processmining.timebasedabstraction.parameters;

import org.deckfour.xes.classification.XEventAndClassifier;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.classification.XEventLifeTransClassifier;
import org.deckfour.xes.classification.XEventNameClassifier;


/**
 * Parameters for the mining of a model from an event log.
 * 
 * @author s. Dündar
 *
 */
public class Parameters {	
	// Intervals
	public static final int MINUTES = 1;
	public static final int HOURS = 2;
	public static final int DAYS = 3;
	public static final int WEEKS = 4;
	public static final int MONTHS = 5;
	
	/**
	 * Classifier parameter. This determines which classifier will be used
	 * during the mining.
	 */
	private XEventClassifier classifier;

	// Default mining mode - 15 minutes
	private int mineMode = MONTHS;
	private int textFieldValue;
	private String comboBoxItem;

	// Create default parameter values.
	public Parameters() {
		classifier = new XEventAndClassifier(new XEventNameClassifier(), new XEventLifeTransClassifier());

	}

	/**
	 * Set the classifier to the given classifier.
	 * 
	 * @param classifier
	 * 					The given classifier.
	 */
	public void setClassifier(XEventClassifier classifier) {
		if (classifier != null) {
			this.classifier = classifier;
		}
	}

	/**
	 * Gets the classifier.
	 * 
	 * @return The classifier.
	 */
	public XEventClassifier getClassifier() {
		return classifier;
	}

	public void setMineMode(int mode) {
		this.mineMode = mode;
	}

	public int getMineMode() {
		return mineMode;
	}
	
	/**
	 * Returns whether these parameters values are equal to the given parameter
	 * values.
	 * 
	 * @param Object
	 * 				The given parameter values.
	 * @return Whether these parameter values are equal to the given parameter
	 * 			values.
	 */
	public boolean equals(Object object) {
		if (object instanceof Parameters) {
			Parameters parameter = (Parameters) object;
			if (classifier.equals(parameter.classifier)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Returns the has code for these parameters.
	 */
	public int hashCode() {
		return classifier.hashCode();
	}

	public void setTextFieldValue(int num) {
		this.textFieldValue = num;		
	}
	
	public int getTextFieldValue() {
		return textFieldValue;
	}

	public void setComboBoxItem(String item) {
		this.comboBoxItem = item;		
	}
	
	public String getComboBoxItem() {
		return comboBoxItem;
	}
}