package org.processmining.timebasedabstraction.plugins;

import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang3.time.DateUtils;
import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.timebasedabstraction.dialogs.Dialog;
import org.processmining.timebasedabstraction.models.TimeAndCountModel;
import org.processmining.timebasedabstraction.parameters.Parameters;

@Plugin(name = "Mine Sessions for Clustering V3",
returnLabels = {"Mine Sessions for Clustering V3"},
returnTypes = {TimeAndCountModel.class},
parameterLabels = {"Log", "Parameters"},
userAccessible = false)
public class TimeAndCountPlugin {

	public static final int TIME_MODE = 0;
	public static final int COUNT_MODE = 1;

	private int mode = 0;

	public void setMode(String mode) {
		if (mode.equals("Time Sessions Clustering")) {
			this.mode = 0;
		} else if (mode.equals("Count Sessions Clustering")) {
			this.mode = 1;
		}
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "S. Dündar", email = "s.dundar@student.tue.nl")
	@PluginVariant(variantLabel = "Mine Sessions, default", requiredParameterLabels = {0} )
	public TimeAndCountModel mineDefault(PluginContext context, XLog log) {
		return mineParameters(context, log, new Parameters());
	}

	public TimeAndCountModel mineDefaultCount(PluginContext context, XLog log) {
		return mineParametersCount(context, log, new Parameters());
	}


	public TimeAndCountModel mineParameters(PluginContext context, XLog log, Parameters parameters) {
		switch (mode) {
			case TIME_MODE:
				TimeAndCountModel timeModel = mineTime(context, log, parameters);
				return timeModel;
			case COUNT_MODE:
				TimeAndCountModel countModel = mineCount(context, log, parameters);
				return countModel;
			default:
				TimeAndCountModel model = mineTime(context, log, parameters);
				return model;
		}
	}

	public TimeAndCountModel mineParametersTime(PluginContext context, XLog log, Parameters parameters) {
		TimeAndCountModel timeModel = mineTime(context, log, parameters);
		return timeModel;
	}

	public TimeAndCountModel mineParametersCount(PluginContext context, XLog log, Parameters parameters) {
		TimeAndCountModel countModel = mineTime(context, log, parameters);
		return countModel;
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "S. Dündar", email = "s.dundar@student.tue.nl")
	@PluginVariant(variantLabel = "Mine a model, dialog", requiredParameterLabels = {0})
	public TimeAndCountModel mineDefault(UIPluginContext context, XLog log) {
		Parameters parameters = new Parameters();
		Dialog dialog = new Dialog(log, parameters);
		InteractionResult result = context.showWizard("Time Based Abstraction", true, true, dialog);
		if (result != InteractionResult.FINISHED) {
			return null;
		}
		return mineParameters(context, log, parameters);
	}

	// MONTH BLOCKS!
	private void mineLogForTimeMonth(TimeAndCountModel model) {
		XLog log = model.getEventLog();
		Map<XEventClass, Double> timeEventMap = new LinkedHashMap<XEventClass, Double>();
		Map<XEventClass, Integer> counterMap = new LinkedHashMap<XEventClass, Integer>();

		model.setTimeEventMap(timeEventMap);
		model.setCounterMap(counterMap);

		double currentEventTime;
		Date currentEventDate;
		double sessionStartTime;
		Date sessionStartDate;
		double sessionEndTime = 0.0d;
		Date sessionEndDate;
		double nextEventTime;
		Date nextEventDate;
		double timeDiff;

		XEvent nextEvent;
		XEvent startSessionEvent;

		for (XTrace trace : log) {
			Object[] traceArray = trace.toArray();
			boolean flag = true;
			for (int i = 0; i < traceArray.length; i++) {
				boolean inBounds = ((i + 1) >= 0) && ((i + 1) < traceArray.length);

				XEvent currentEvent = (XEvent) traceArray[i];
				if (inBounds) {
					nextEvent = (XEvent) traceArray[i + 1];
				} else {
					nextEvent = null;
				}

				currentEventTime = XTimeExtension.instance().extractTimestamp(currentEvent).getTime();
				if (nextEvent == null) {
					nextEventTime = 0.0d;
				} else {
					nextEventTime = XTimeExtension.instance().extractTimestamp(nextEvent).getTime();
				}
				timeDiff = nextEventTime - currentEventTime;

				if (flag) {
					startSessionEvent = (XEvent) traceArray[0];
					sessionStartDate = XTimeExtension.instance().extractTimestamp(startSessionEvent);
					sessionStartTime = sessionStartDate.getTime();
					sessionEndDate = DateUtils.addMonths(sessionStartDate, 1);
					sessionEndTime = sessionEndDate.getTime();
					flag = false;
				}

				if (nextEventTime <= sessionEndTime && timeDiff >= 0) {
					model.totalTimeEventMap(currentEvent, timeDiff);
					model.addToCounterMap(currentEvent, 1);
				} else {
					model.totalTimeEventMap(currentEvent, 0.0d);
					model.addToCounterMap(currentEvent, 1);
				}

				if (nextEvent != null) {
					startSessionEvent = nextEvent;
					sessionStartDate = XTimeExtension.instance().extractTimestamp(startSessionEvent);
					sessionStartTime = sessionStartDate.getTime();
					sessionEndDate = DateUtils.addMonths(sessionStartDate, 1);
					sessionEndTime = sessionEndDate.getTime();
				} else {
					break;
				}
			}
		}
	}
	//	 15 MIN BLOCKS!
	private void mineLogForTime15Block(TimeAndCountModel model) {
		XLog log = model.getEventLog();
		Map<XEventClass, Double> timeEventMap = new LinkedHashMap<XEventClass, Double>();
		Map<XEventClass, Integer> counterMap = new LinkedHashMap<XEventClass, Integer>();

		model.setTimeEventMap(timeEventMap);
		model.setCounterMap(counterMap);

		double currentEventTime;
		double sessionStartTime;
		double sessionEndTime = 0.0d;
		double nextEventTime;
		double timeDiff;

		XEvent nextEvent;
		XEvent startSessionEvent;

		for (XTrace trace : log) {
			Object[] traceArray = trace.toArray();
			boolean flag = true;
			for (int i = 0; i < traceArray.length; i++) {
				boolean inBounds = ((i + 1) >= 0) && ((i + 1) < traceArray.length);

				XEvent currentEvent = (XEvent) traceArray[i];
				if (inBounds) {
					nextEvent = (XEvent) traceArray[i + 1];
				} else {
					nextEvent = null;
				}

				currentEventTime = XTimeExtension.instance().extractTimestamp(currentEvent).getTime();
				if (nextEvent == null) {
					nextEventTime = 0.0d;
				} else {
					nextEventTime = XTimeExtension.instance().extractTimestamp(nextEvent).getTime();
				}
				timeDiff = nextEventTime - currentEventTime;

				if (flag) {
					startSessionEvent = (XEvent) traceArray[0];
					sessionStartTime = XTimeExtension.instance().extractTimestamp(startSessionEvent).getTime();
					sessionEndTime = sessionStartTime + TimeUnit.MINUTES.toMillis(15);
					flag = false;
				}

				if (nextEventTime <= sessionEndTime && timeDiff >= 0) {
					model.totalTimeEventMap(currentEvent, timeDiff);
					model.addToCounterMap(currentEvent, 1);
				} else {
					model.totalTimeEventMap(currentEvent, 0.0d);
					model.addToCounterMap(currentEvent, 1);
				}

				if (nextEvent != null) {
					startSessionEvent = nextEvent;
					sessionStartTime = XTimeExtension.instance().extractTimestamp(startSessionEvent).getTime();
					sessionEndTime = sessionStartTime + TimeUnit.MINUTES.toMillis(15);

				} else {
					break;
				}
			}
		}
	}

	//	 IF TIMEDIFF > 15 MIN.
	private void mineLogForTime15(TimeAndCountModel model) {
		XLog log = model.getEventLog();
		Map<XEventClass, Double> timeEventMap = new LinkedHashMap<XEventClass, Double>();
		Map<XEventClass, Integer> counterMap = new LinkedHashMap<XEventClass, Integer>();

		model.setTimeEventMap(timeEventMap);
		model.setCounterMap(counterMap);

		double currentEventTime;
		double nextEventTime;
		double timeDiff;

		XEvent nextEvent;

		for (XTrace trace : log) {
			Object[] traceArray = trace.toArray();
			for (int i = 0; i < traceArray.length; i++) {
				boolean inBounds = ((i + 1) >= 0) && ((i + 1) < traceArray.length);

				XEvent currentEvent = (XEvent) traceArray[i];
				if (inBounds) {
					nextEvent = (XEvent) traceArray[i + 1];
				} else {
					nextEvent = null;
				}

				currentEventTime = XTimeExtension.instance().extractTimestamp(currentEvent).getTime();
				if (nextEvent == null) {
					nextEventTime = 0.0d;
				} else {
					nextEventTime = XTimeExtension.instance().extractTimestamp(nextEvent).getTime();
				}
				timeDiff = nextEventTime - currentEventTime;

				if (timeDiff >= 0 && timeDiff < TimeUnit.MINUTES.toMillis(15)) {
					model.totalTimeEventMap(currentEvent, timeDiff);
					model.addToCounterMap(currentEvent, 1);
				} else {
					model.totalTimeEventMap(currentEvent, 0.0d);
					model.addToCounterMap(currentEvent, 1);
				}
			}
		}
	}

	// BLOCKS OF MONTHS!
	//	private TimeAndCountModel mineTime(PluginContext context, XLog log, Parameters parameters) {
	//		context.getProgress().setMaximum(log.size());
	//		XLogInfo info = XLogInfoFactory.createLogInfo(log, parameters.getClassifier());
	//		TimeAndCountModel model = new TimeAndCountModel();
	//		model.setEventLog(log);
	//		model.setEventClasses(info);
	//		model.setKeys();
	//		Map<XEventClass, Double> eventMap;
	//		double currentEventTime;
	//		Date currentEventDate;
	//		double sessionStartTime;
	//		Date sessionStartDate;
	//		double sessionEndTime = 0.0d;
	//		Date sessionEndDate;
	//		double nextEventTime;
	//		Date nextEventDate;
	//		double timeDiff;
	//		XEvent nextEvent;
	//		XEvent startSessionEvent;
	//
	//		mineLogForTimeMonth(model);
	//
	//		for (XTrace trace : log) {
	//			eventMap = new LinkedHashMap<XEventClass, Double>();
	//			model.setEventMap(eventMap);
	//			Object[] traceArray = trace.toArray();
	//			boolean flag = true;
	//			for (int i = 0; i < traceArray.length; i++) {
	//				boolean inBounds = ((i + 1) >= 0) && ((i + 1) < traceArray.length);
	//
	//				XEvent currentEvent = (XEvent) traceArray[i];
	//				currentEventTime = XTimeExtension.instance().extractTimestamp(currentEvent).getTime();
	//
	//				if (inBounds) {
	//					nextEvent = (XEvent) traceArray[i + 1];
	//				} else {
	//					nextEvent = null;
	//				}
	//
	//				if (nextEvent == null) {
	//					nextEventTime = 0.0d;
	//				} else {
	//					nextEventTime = XTimeExtension.instance().extractTimestamp(nextEvent).getTime();
	//				}
	//				timeDiff = nextEventTime - currentEventTime;
	//
	//				if (flag) {
	//					startSessionEvent = (XEvent) traceArray[0];
	//					sessionStartDate = XTimeExtension.instance().extractTimestamp(startSessionEvent);
	//					sessionStartTime = sessionStartDate.getTime();
	//					sessionEndDate = DateUtils.addMonths(sessionStartDate, 1);
	//					sessionEndTime = sessionEndDate.getTime();
	//					flag = false;
	//				}
	//
	//				if (nextEventTime <= sessionEndTime && timeDiff >= 0) {
	//					model.addToEventMap(currentEvent, timeDiff);
	//				} else {
	//					double avg = (model.getTimeEventMap().get(model.getEventClasses().getClassOf(currentEvent))) / (model.getCounterMap().get(model.getEventClasses().getClassOf(currentEvent)));
	//					model.addToEventMap(currentEvent, avg);
	//
	//					if (!model.getEventMap().isEmpty()) {
	//						model.addToEventMapList(eventMap);
	//					}
	//
	//					if (nextEvent != null) {
	//						startSessionEvent = nextEvent;
	//						sessionStartDate = XTimeExtension.instance().extractTimestamp(startSessionEvent);
	//						sessionStartTime = sessionStartDate.getTime();
	//						sessionEndDate = DateUtils.addMonths(sessionStartDate, 1);
	//						sessionEndTime = sessionEndDate.getTime();	                    
	//					} else {
	//						break;
	//					}
	//
	//					eventMap = new LinkedHashMap<XEventClass, Double>();
	//					model.setEventMap(eventMap);
	//				}
	//			}
	//			context.getProgress().inc();
	//		}
	//
	//		// Normalize the data
	//		model.normalizeData();
	//
	//		// Create the Weka dataset
	//		model.createDataSet();
	//
	//		return model;
	//	}

	// MINE IN BLOCKS OF 15 MINUTES 
	//	private TimeAndCountModel mineTime(PluginContext context, XLog log, Parameters parameters) {
	//		context.getProgress().setMaximum(log.size());
	//		XLogInfo info = XLogInfoFactory.createLogInfo(log, parameters.getClassifier());
	//		TimeAndCountModel model = new TimeAndCountModel();
	//		model.setEventLog(log);
	//		model.setEventClasses(info);
	//		model.setKeys();
	//		Map<XEventClass, Double> eventMap;
	//		double currentEventTime;
	//		double sessionStartTime;
	//		double sessionEndTime = 0.0d;
	//		double nextEventTime;
	//		double timeDiff;
	//		XEvent nextEvent;
	//		XEvent startSessionEvent;
	//
	//		mineLogForTime15Block(model);
	//
	//		for (XTrace trace : log) {
	//			eventMap = new LinkedHashMap<XEventClass, Double>();
	//			model.setEventMap(eventMap);
	//			Object[] traceArray = trace.toArray();
	//			boolean flag = true;
	//			for (int i = 0; i < traceArray.length; i++) {
	//				boolean inBounds = ((i + 1) >= 0) && ((i + 1) < traceArray.length);
	//
	//				XEvent currentEvent = (XEvent) traceArray[i];
	//				currentEventTime = XTimeExtension.instance().extractTimestamp(currentEvent).getTime();
	//
	//				if (inBounds) {
	//					nextEvent = (XEvent) traceArray[i + 1];
	//				} else {
	//					nextEvent = null;
	//				}
	//
	//				if (nextEvent == null) {
	//					nextEventTime = 0.0d;
	//				} else {
	//					nextEventTime = XTimeExtension.instance().extractTimestamp(nextEvent).getTime();
	//				}
	//				timeDiff = nextEventTime - currentEventTime;
	//
	//				if (flag) {
	//					startSessionEvent = (XEvent) traceArray[0];
	//					sessionStartTime = XTimeExtension.instance().extractTimestamp(startSessionEvent).getTime();
	//					sessionEndTime = sessionStartTime + TimeUnit.MINUTES.toMillis(15);
	//					flag = false;
	//				}
	//
	//				if (nextEventTime <= sessionEndTime && timeDiff >= 0) {
	//					model.addToEventMap(currentEvent, timeDiff);
	//				} else {
	//					//					if (model.getEventClasses().getClassOf(currentEvent).toString().equals("Visit page home+complete")) {
	//					//						model.addToEventMap(currentEvent, TimeUnit.SECONDS.toMillis(50));
	//					//					} else {
	//					double avg = (model.getTimeEventMap().get(model.getEventClasses().getClassOf(currentEvent))) / (model.getCounterMap().get(model.getEventClasses().getClassOf(currentEvent)));
	//					model.addToEventMap(currentEvent, avg);
	//					//					}
	//
	//					if (!model.getEventMap().isEmpty()) {
	//						model.addToEventMapList(eventMap);
	//					}
	//
	//					if (nextEvent != null) {
	//						startSessionEvent = nextEvent;
	//						sessionStartTime = XTimeExtension.instance().extractTimestamp(startSessionEvent).getTime();
	//						sessionEndTime = sessionStartTime + TimeUnit.MINUTES.toMillis(15);
	//
	//					} else {
	//						break;
	//					}
	//					eventMap = new LinkedHashMap<XEventClass, Double>();
	//					model.setEventMap(eventMap);
	//				}
	//			}
	//			context.getProgress().inc();
	//		}
	//
	//		// Normalize the data
	//		model.normalizeData();
	//
	//		// Create the Weka dataset
	//		model.createDataSet();
	//
	//		return model;
	//	}

	// IF TIMEDIFF > 15 THEN THE SESSION HAS ENDED
	private TimeAndCountModel mineTime(PluginContext context, XLog log, Parameters parameters) {
		context.getProgress().setMaximum(log.size());
		XLogInfo info = XLogInfoFactory.createLogInfo(log, parameters.getClassifier());
		TimeAndCountModel model = new TimeAndCountModel();
		model.setEventLog(log);
		model.setEventClasses(info);
		model.setKeys();
		Map<XEventClass, Double> eventMap;
		double currentEventTime;
		double nextEventTime;
		double timeDiff;
		XEvent nextEvent;

		mineLogForTime15(model);

		for (XTrace trace : log) {
			eventMap = new LinkedHashMap<XEventClass, Double>();
			model.setEventMap(eventMap);
			Object[] traceArray = trace.toArray();
			for (int j = 0; j < traceArray.length; j++) {
				boolean inBounds = ((j + 1) >= 0) && ((j+1) < traceArray.length);

				XEvent currentEvent = (XEvent) traceArray[j];
				if (inBounds) {
					nextEvent = (XEvent) traceArray[j + 1];
				} else {
					nextEvent = null;
				}

				currentEventTime = XTimeExtension.instance().extractTimestamp(currentEvent).getTime();				
				if (nextEvent == null) {
					nextEventTime = 0.0d;
				} else {
					nextEventTime = XTimeExtension.instance().extractTimestamp(nextEvent).getTime();	
				}				
				timeDiff = nextEventTime - currentEventTime;

				if (timeDiff >= 0 && timeDiff < TimeUnit.MINUTES.toMillis(15)) {
					model.addToEventMap(currentEvent, timeDiff);
				} else {
					//					if (model.getEventClasses().getClassOf(currentEvent).toString().equals("Visit page home+complete")) {
					//						model.addToEventMap(currentEvent, 0.0d);
					//					} else {
					double avg = (model.getTimeEventMap().get(model.getEventClasses().getClassOf(currentEvent))) / (model.getCounterMap().get(model.getEventClasses().getClassOf(currentEvent)));
					model.addToEventMap(currentEvent, avg);
					//					}

					if (!model.getEventMap().isEmpty()) {
						model.addToEventMapList(eventMap);
					}

					eventMap = new LinkedHashMap<XEventClass, Double>();
					model.setEventMap(eventMap);
				}
			}
			context.getProgress().inc();
		}

		// Normalize the data
		model.normalizeData();

		// Create the Weka dataset
		model.createDataSet();

		return model;
	}

	private TimeAndCountModel mineCount(PluginContext context, XLog log, Parameters parameters) {
		context.getProgress().setMaximum(log.size());

		XLogInfo info = XLogInfoFactory.createLogInfo(log, parameters.getClassifier());
		TimeAndCountModel model = new TimeAndCountModel();

		model.setEventLog(log);
		model.setEventClasses(info);
		model.setKeys();

		Map<XEventClass, Double> eventMap;

		double currentTime;
		double eventTime;

		for (XTrace trace : log) {
			eventMap = new LinkedHashMap<XEventClass, Double>();
			model.setEventMap(eventMap);
			currentTime = 0;
			boolean first = true;
			for (XEvent event : trace) {
				eventTime = XTimeExtension.instance().extractTimestamp(event).getTime();
				if (first) {
					currentTime = eventTime;
					first = false;
				}
				if (eventTime < currentTime + TimeUnit.MINUTES.toMillis(15)) {
					model.addToEventMap(event, 1);
				} else {
					if (!model.getEventMap().isEmpty()) {
						model.addToEventMapList(eventMap);
					}

					eventMap = new LinkedHashMap<XEventClass, Double>();
					model.setEventMap(eventMap);
					model.addToEventMap(event, 1);
				}

				currentTime = eventTime;
			}
			model.addToEventMapList(eventMap);

			context.getProgress().inc();
		}

		// Normalize the data
		model.normalizeData();

		// Create the Weka dataset
		model.createDataSet();

		return model;
	}



}
