package org.processmining.timebasedabstraction.plugins;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.timebasedabstraction.dialogs.Dialog;
import org.processmining.timebasedabstraction.mineCount.MineDaysCount;
import org.processmining.timebasedabstraction.mineCount.MineHoursCount;
import org.processmining.timebasedabstraction.mineCount.MineMinutesCount;
import org.processmining.timebasedabstraction.mineCount.MineMonthsCount;
import org.processmining.timebasedabstraction.mineCount.MineWeeksCount;
import org.processmining.timebasedabstraction.mineTime.MineDays;
import org.processmining.timebasedabstraction.mineTime.MineHours;
import org.processmining.timebasedabstraction.mineTime.MineMinutes;
import org.processmining.timebasedabstraction.mineTime.MineMonths;
import org.processmining.timebasedabstraction.mineTime.MineWeeks;
import org.processmining.timebasedabstraction.models.TimeAndCountModel;
import org.processmining.timebasedabstraction.parameters.Parameters;

@Plugin(name = "Mine Sessions for Clustering V4",
returnLabels = {"Mine Sessions for Clustering V4"},
returnTypes = {TimeAndCountModel.class},
parameterLabels = {"Log", "Parameters"},
userAccessible = false)
public class TimeAndCountPlugin2 {

	public static final int TIME_MODE = 0;
	public static final int COUNT_MODE = 1;

	private int mode = 0;

	public void setMode(String mode) {
		if (mode.equals("Time Sessions Clustering")) {
			this.mode = 0;
		} else if (mode.equals("Count Sessions Clustering")) {
			this.mode = 1;
		}
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "S. Dündar", email = "s.dundar@student.tue.nl")
	@PluginVariant(variantLabel = "Mine Sessions, default", requiredParameterLabels = {0} )
	public TimeAndCountModel mineDefault(PluginContext context, XLog log) {
		return mineParameters(context, log, new Parameters());
	}

	public TimeAndCountModel mineParameters(PluginContext context, XLog log, Parameters parameters) {
		switch (mode) {
			case TIME_MODE:
				if (parameters.getMineMode() == parameters.MINUTES) {
					MineMinutes mineMinutes = new MineMinutes();
					TimeAndCountModel timeModelMinutes = mineMinutes.mineTime(context, log, parameters);
					timeModelMinutes.setParameterValues(parameters);
					return timeModelMinutes;
				} else if (parameters.getMineMode() == parameters.HOURS) {
					MineHours mineHours = new MineHours();
					TimeAndCountModel timeModelHours = mineHours.mineTime(context, log, parameters);
					timeModelHours.setParameterValues(parameters);
					return timeModelHours;
				} else if (parameters.getMineMode() == parameters.DAYS) {
					MineDays mineDays = new MineDays();
					TimeAndCountModel timeModelDays = mineDays.mineTime(context, log, parameters);
					timeModelDays.setParameterValues(parameters);
					return timeModelDays;
				} else if (parameters.getMineMode() == parameters.WEEKS) {
					MineWeeks mineWeeks = new MineWeeks();
					TimeAndCountModel timeModelWeeks = mineWeeks.mineTime(context, log, parameters);
					timeModelWeeks.setParameterValues(parameters);
					return timeModelWeeks;
				} else if (parameters.getMineMode() == parameters.MONTHS) {
					MineMonths mineMonths = new MineMonths();
					TimeAndCountModel timeModelMonths = mineMonths.mineTime(context, log, parameters);
					timeModelMonths.setParameterValues(parameters);
					return timeModelMonths;
				}
				break;
			case COUNT_MODE:
				if (parameters.getMineMode() == parameters.MINUTES) {
					MineMinutesCount mineMinutesCount = new MineMinutesCount();
					TimeAndCountModel countModelMinutes = mineMinutesCount.mineCount(context, log, parameters);
					countModelMinutes.setParameterValues(parameters);
					return countModelMinutes;
				} else if (parameters.getMineMode() == parameters.MINUTES) {
					MineHoursCount mineHoursCount = new MineHoursCount();
					TimeAndCountModel countModelHours = mineHoursCount.mineCount(context, log, parameters);
					countModelHours.setParameterValues(parameters);
					return countModelHours;
				} else if (parameters.getMineMode() == parameters.MINUTES) {
					MineDaysCount mineDaysCount = new MineDaysCount();
					TimeAndCountModel countModelDays = mineDaysCount.mineCount(context, log, parameters);
					countModelDays.setParameterValues(parameters);
					return countModelDays;
				} else if (parameters.getMineMode() == parameters.WEEKS) {
					MineWeeksCount mineWeeksCount = new MineWeeksCount();
					TimeAndCountModel countModelWeeks = mineWeeksCount.mineCount(context, log, parameters);
					countModelWeeks.setParameterValues(parameters);
					return countModelWeeks;
				} else if (parameters.getMineMode() == parameters.MONTHS) {
					MineMonthsCount mineMonthsCount = new MineMonthsCount();
					TimeAndCountModel countModelMonths = mineMonthsCount.mineCount(context, log, parameters);
					countModelMonths.setParameterValues(parameters);
					return countModelMonths;
				}
				break;
			default:
//				Mine15Blocks mine15Blocks = new Mine15Blocks();
//				TimeAndCountModel timeModel15MinBlock = mine15Blocks.mineTime(context, log, parameters);
//				timeModel15MinBlock.setParameterValues(parameters);
//				return timeModel15MinBlock;
		}
		return null;
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "S. Dündar", email = "s.dundar@student.tue.nl")
	@PluginVariant(variantLabel = "Mine a model, dialog", requiredParameterLabels = {0})
	public TimeAndCountModel mineDefault(UIPluginContext context, XLog log) {
		Parameters parameters = new Parameters();
		Dialog dialog = new Dialog(log, parameters);
		InteractionResult result = context.showWizard("Time Based Abstraction", true, true, dialog);
		if (result == InteractionResult.FINISHED) {
			return mineParameters(context, log, parameters);
		}
		return null;
	}


}
