package org.processmining.timebasedabstraction.plugins;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.timebasedabstraction.mineCount.MineDaysCount;
import org.processmining.timebasedabstraction.mineCount.MineHoursCount;
import org.processmining.timebasedabstraction.mineCount.MineMinutesCount;
import org.processmining.timebasedabstraction.mineCount.MineMonthsCount;
import org.processmining.timebasedabstraction.mineCount.MineWeeksCount;
import org.processmining.timebasedabstraction.mineTime.MineDays;
import org.processmining.timebasedabstraction.mineTime.MineHours;
import org.processmining.timebasedabstraction.mineTime.MineMinutes;
import org.processmining.timebasedabstraction.mineTime.MineMonths;
import org.processmining.timebasedabstraction.mineTime.MineWeeks;
import org.processmining.timebasedabstraction.models.TimeAndCountModel;
import org.processmining.timebasedabstraction.parameters.Parameters;

@Plugin(name = "Session-based Log Abstraction",
returnLabels = {"Session-based Log Abstraction"},
returnTypes = {TimeAndCountModel.class},
parameterLabels = {"Log", "Parameters"},
userAccessible = true)
public class TimeAndCountPlugin3 {
	public static final int TIME_MODE = 0;
	public static final int COUNT_MODE = 1;

	private int mode = 0;

	public void setMode(String mode) {
		if (mode.equals("Clustering Sessions Based on Time")) {
			this.mode = TIME_MODE;
		} else if (mode.equals("Clustering Sessions Based on Frequency")) {
			this.mode = COUNT_MODE;
		}
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "M. de Leoni, S. Dundar", email = "m.d.leoni@tue.nl")
	@PluginVariant(variantLabel = "Mine Sessions, default", requiredParameterLabels = {0} )
	public TimeAndCountModel mineDefault(PluginContext context, XLog log) {
		return mineParameters(context, log, new Parameters());
	}

	public TimeAndCountModel mineParameters(PluginContext context, XLog log, Parameters parameters) {
		switch (mode) {
			case TIME_MODE:
//				if (parameters.getMineMode() == parameters.MIN15) {
//					Mine15 mine15 = new Mine15();
//					TimeAndCountModel timeModel15Min = mine15.mineTime(context, log, parameters);
//					timeModel15Min.setParameterValues(parameters);
//					return timeModel15Min;
//				} else if (parameters.getMineMode() == parameters.DAILY) {
//					MineDaily mineDaily = new MineDaily();
//					TimeAndCountModel timeModelDaily = mineDaily.mineTime(context, log, parameters);
//					timeModelDaily.setParameterValues(parameters);
//					return timeModelDaily;
//				} else if (parameters.getMineMode() == parameters.WEEKLY) {
//					MineWeekly mineWeekly = new MineWeekly();
//					TimeAndCountModel timeModelWeekly = mineWeekly.mineTime(context, log, parameters);
//					timeModelWeekly.setParameterValues(parameters);
//					return timeModelWeekly;
//				} else if (parameters.getMineMode() == parameters.MONTHLY) {
//					MineMonthly mineMonth = new MineMonthly();
//					TimeAndCountModel timeModelMonth = mineMonth.mineTime(context, log, parameters);
//					timeModelMonth.setParameterValues(parameters);
//					return timeModelMonth;
//				} else 
				if (parameters.getMineMode() == parameters.MINUTES) {
					MineMinutes mineMinutes = new MineMinutes();
					TimeAndCountModel timeModelMinutes = mineMinutes.mineTime(context, log, parameters);
					timeModelMinutes.setParameterValues(parameters);
					return timeModelMinutes;
				} else if (parameters.getMineMode() == parameters.HOURS) {
					MineHours mineHours = new MineHours();
					TimeAndCountModel timeModelHours = mineHours.mineTime(context, log, parameters);
					timeModelHours.setParameterValues(parameters);
					return timeModelHours;
				} else if (parameters.getMineMode() == parameters.DAYS) {
					MineDays mineDays = new MineDays();
					TimeAndCountModel timeModelDays = mineDays.mineTime(context, log, parameters);
					timeModelDays.setParameterValues(parameters);
					return timeModelDays;
				} else if (parameters.getMineMode() == parameters.WEEKS) {
					MineWeeks mineWeeks = new MineWeeks();
					TimeAndCountModel timeModelWeeks = mineWeeks.mineTime(context, log, parameters);
					timeModelWeeks.setParameterValues(parameters);
					return timeModelWeeks;
				} else if (parameters.getMineMode() == parameters.MONTHS) {
					MineMonths mineMonths = new MineMonths();
					TimeAndCountModel timeModelMonths = mineMonths.mineTime(context, log, parameters);
					timeModelMonths.setParameterValues(parameters);
					return timeModelMonths;
				}
				break;
			case COUNT_MODE:
//				if (parameters.getMineMode() == parameters.MIN15) {
//					Mine15Count mine15Count = new Mine15Count();
//					TimeAndCountModel countModel15Min = mine15Count.mineCount(context, log, parameters);
//					countModel15Min.setParameterValues(parameters);
//					return countModel15Min;
//				} else if (parameters.getMineMode() == parameters.DAILY) {
//					MineDailyCount mineDailyCount = new MineDailyCount();
//					TimeAndCountModel countModelDaily = mineDailyCount.mineCount(context, log, parameters);
//					countModelDaily.setParameterValues(parameters);
//					return countModelDaily;
//				} else if (parameters.getMineMode() == parameters.WEEKLY) {
//					MineWeeklyCount mineWeeklyCount = new MineWeeklyCount();
//					TimeAndCountModel countModelWeekly = mineWeeklyCount.mineCount(context, log, parameters);
//					countModelWeekly.setParameterValues(parameters);
//					return countModelWeekly;
//				} else if (parameters.getMineMode() == parameters.MONTHLY) {
//					MineMonthlyCount mineMonthlyCount = new MineMonthlyCount();
//					TimeAndCountModel countModelMonthly = mineMonthlyCount.mineCount(context, log, parameters);
//					countModelMonthly.setParameterValues(parameters);
//					return countModelMonthly;
//				} else 
				if (parameters.getMineMode() == parameters.MINUTES) {
					MineMinutesCount mineMinutesCount = new MineMinutesCount();
					TimeAndCountModel countModelMinutes = mineMinutesCount.mineCount(context, log, parameters);
					countModelMinutes.setParameterValues(parameters);
					return countModelMinutes;
				} else if (parameters.getMineMode() == parameters.HOURS) {
					MineHoursCount mineHoursCount = new MineHoursCount();
					TimeAndCountModel countModelHours = mineHoursCount.mineCount(context, log, parameters);
					countModelHours.setParameterValues(parameters);
					return countModelHours;
				} else if (parameters.getMineMode() == parameters.DAYS) {
					MineDaysCount mineDaysCount = new MineDaysCount();
					TimeAndCountModel countModelDays = mineDaysCount.mineCount(context, log, parameters);
					countModelDays.setParameterValues(parameters);
					return countModelDays;
				} else if (parameters.getMineMode() == parameters.WEEKS) {
					MineWeeksCount mineWeeksCount = new MineWeeksCount();
					TimeAndCountModel countModelWeeks = mineWeeksCount.mineCount(context, log, parameters);
					countModelWeeks.setParameterValues(parameters);
					return countModelWeeks;
				} else if (parameters.getMineMode() == parameters.MONTHS) {
					MineMonthsCount mineMonthsCount = new MineMonthsCount();
					TimeAndCountModel countModelMonths = mineMonthsCount.mineCount(context, log, parameters);
					countModelMonths.setParameterValues(parameters);
					return countModelMonths;
				}
				break;
			default:
				return null;
//				Mine15 mine15 = new Mine15();
//				TimeAndCountModel timeModel15Min = mine15.mineTime(context, log, parameters);
//				timeModel15Min.setParameterValues(parameters);
//				return timeModel15Min;
		}
		return null;
	}
	
}
