package org.processmining.timebasedabstraction.visualization;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.MouseInfo;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;
import java.util.concurrent.Executor;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.Timer;

import org.deckfour.uitopia.api.model.ResourceType;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.processmining.contexts.uitopia.UIContext;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.contexts.uitopia.model.ProMPOResource;
import org.processmining.contexts.uitopia.model.ProMTask;
import org.processmining.framework.plugin.GlobalContext;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.providedobjects.ProvidedObjectID;
import org.processmining.plugins.utils.ProvidedObjectHelper;
import org.processmining.timebasedabstraction.algorithms.DbscanElki;
import org.processmining.timebasedabstraction.algorithms.ElbowMethod;
import org.processmining.timebasedabstraction.algorithms.KMeans;
import org.processmining.timebasedabstraction.models.TimeAndCountModel;
import org.processmining.timebasedabstraction.parameters.Parameters;
import org.processmining.timebasedabstraction.plugins.CreateAbstractedLog;
import org.processmining.timebasedabstraction.plugins.TimeAndCountPlugin3;
import org.tc33.jheatchart.HeatChart;

import com.fluxicon.slickerbox.components.NiceDoubleSlider;
import com.fluxicon.slickerbox.components.NiceIntegerSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;

public class SessionsVisualizer {
	UIPluginContext globalContext;
	TimeAndCountModel globalModel;
	Parameters globalParameter;

	private SlickerFactory slickerFactory;

	private JPanel headPanel;
	private JPanel leftPane;

	JSplitPane entireView = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);

	JTabbedPane tabbedPane;

	// Variables declaration - do not modify                     
	private javax.swing.JComboBox<String> algorithmCb;
	private javax.swing.JLabel clusteringAlgorithmLabel;
	private javax.swing.JComboBox<String> clusteringModeCb;
	private javax.swing.JLabel clusteringModeLabel;
	private javax.swing.JLabel clusteringSettingsLabel;
	private javax.swing.JButton elbowGraphButton;
	private NiceDoubleSlider epsSlider;
	private javax.swing.JLabel heatmapDBSCANLabel;
	private javax.swing.JLabel heatmapKmeansLabel;
	private javax.swing.JSeparator jSeparator1;
	private javax.swing.JSeparator jSeparator2;
	private javax.swing.JSeparator jSeparator3;
	private NiceIntegerSlider minPointsSlider;
	private javax.swing.JComboBox<String> numClustersCb;
	private javax.swing.JLabel numClustersLabel;
	private javax.swing.JLabel parameterValueLabel;
	private NiceIntegerSlider parameterValueSlider;
	private javax.swing.JButton startButton;
	private javax.swing.JButton startClusteringButton;
	private javax.swing.JComboBox<String> timeCb;
	private javax.swing.JCheckBox noiseToClusterCheckbox;
	// End of variables declaration

	private ChartPanel chartPanelTime;
	private ChartPanel chartPanelCount;
	private ChartDialog chartDialog;

	private ImageIcon icon;
	private JLabel imageLabel;

	public Map<Integer, String> clusterNameMap;
	public Map<Integer, Integer> indexToClusterMap;

	double[] sqErrorsTime;
	double[] sqErrorsCount;
	int numberOfTabs = 0;
	boolean countFlag = false;
	boolean timeFlag = false;
	boolean publishNumberedFlag = false;
	boolean noiseToClusterFlag = false;

	ElbowMethod elbow = new ElbowMethod();
	HeatChart heatmap;

	@Plugin(name = "Show Cluster Visualization", returnLabels = {"Cluster Visualization"},
			returnTypes = {JComponent.class}, parameterLabels = {"TimeAndCountModel"},
			userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "S. Dündar", email = "s.dundar@student.tue.nl")
	@Visualizer(name = "Time -and Frequency-Based Abstraction")
	public JComponent visualize(UIPluginContext context, TimeAndCountModel model) throws Exception {
		globalContext = context;
		globalModel = model;
		globalParameter = globalModel.getParameterValues();

		slickerFactory = SlickerFactory.instance();

		// Return the results in headpanel
		headPanel = new JPanel(new BorderLayout());
		entireView.setOpaque(false);
		entireView.setOneTouchExpandable(true);

		// Create the JTabbedPane and set it as the right component of the entire splitpane
		tabbedPane = new JTabbedPane();
		entireView.setRightComponent(tabbedPane);

		// Left panel: Parameters selection and calculate button
		leftPane = new JPanel();
		leftPane.setPreferredSize(new Dimension(250, 80));
		leftPane.setBackground(Color.LIGHT_GRAY);
		leftPane.setForeground(Color.BLACK);

		clusteringSettingsLabel = new javax.swing.JLabel();
		jSeparator1 = new javax.swing.JSeparator();
		heatmapKmeansLabel = new javax.swing.JLabel();
		clusteringModeLabel = new javax.swing.JLabel();
		clusteringModeCb = new javax.swing.JComboBox<>();
		clusteringAlgorithmLabel = new javax.swing.JLabel();
		algorithmCb = new javax.swing.JComboBox<>();
		parameterValueLabel = new javax.swing.JLabel();
		parameterValueSlider = slickerFactory.createNiceIntegerSlider("", 0, 60, 15, Orientation.HORIZONTAL);
		timeCb = new javax.swing.JComboBox<>();
		startClusteringButton = new javax.swing.JButton();
		noiseToClusterCheckbox = new javax.swing.JCheckBox("Assign Noisy Points to Clusters", false);
		noiseToClusterCheckbox.setBackground(leftPane.getBackground());
		noiseToClusterCheckbox.setForeground(leftPane.getForeground());

		numClustersLabel = new javax.swing.JLabel();
		Integer[] clusterNumbs = new Integer[20];
		for (int i = 0; i < clusterNumbs.length; i++) {
			clusterNumbs[i] = i + 1;
		}
		numClustersCb = new JComboBox(clusterNumbs);

		elbowGraphButton = slickerFactory.createButton("Elbow Graph");
		jSeparator2 = new javax.swing.JSeparator();
		heatmapDBSCANLabel = new javax.swing.JLabel();
		epsSlider = slickerFactory.createNiceDoubleSlider("Eps", 0.0, 1, 0.25, Orientation.HORIZONTAL);
		XLogInfo info = XLogInfoFactory.createLogInfo(globalModel.getEventLog());		
		minPointsSlider = slickerFactory.createNiceIntegerSlider("MinPts", 1, info.getEventClasses().size(), info.getEventClasses().size()/2, Orientation.HORIZONTAL);
		startButton = new javax.swing.JButton();
		jSeparator3 = new javax.swing.JSeparator();

		clusteringSettingsLabel.setFont(new java.awt.Font("Tahoma", 1, 18)); // NOI18N
		clusteringSettingsLabel.setText("Clustering Settings");
		clusteringSettingsLabel.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);

		heatmapKmeansLabel.setFont(new java.awt.Font("Tahoma", 1, 18)); // NOI18N
		heatmapKmeansLabel.setText("Heatmap Settings K-Means");
		heatmapKmeansLabel.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);

		clusteringModeLabel.setFont(new java.awt.Font("Tahoma", 1, 13)); // NOI18N
		clusteringModeLabel.setText("Select Encoding Mode");

		clusteringModeCb.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "Encoding Sessions Based on Time", "Encoding Sessions Based on Frequency" }));

		clusteringAlgorithmLabel.setFont(new java.awt.Font("Tahoma", 1, 13)); // NOI18N
		clusteringAlgorithmLabel.setText("Select Clustering Algorithm");

		algorithmCb.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "K-Means", "DBSCAN" }));

		parameterValueLabel.setFont(new java.awt.Font("Tahoma", 1, 13)); // NOI18N
		parameterValueLabel.setText("Set Time Interval");

		timeCb.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "Minutes", "Hours", "Days", "Weeks", "Months" }));

		startClusteringButton.setText("Encode Sessions");

		numClustersLabel.setFont(new java.awt.Font("Tahoma", 1, 13)); // NOI18N
		numClustersLabel.setText("Select Number Of Clusters");

		elbowGraphButton.setText("Elbow Graph");

		heatmapDBSCANLabel.setFont(new java.awt.Font("Tahoma", 1, 18)); // NOI18N
		heatmapDBSCANLabel.setText("Heatmap Settings DBSCAN");

		startButton.setText("Generate Heatmap");

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(leftPane);
		leftPane.setLayout(layout);
		layout.setHorizontalGroup(
				layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(layout.createSequentialGroup()
						.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
								.addGroup(layout.createSequentialGroup()
										.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
												.addGroup(layout.createSequentialGroup()
														.addGap(77, 77, 77)
														.addComponent(startClusteringButton))
												.addGroup(layout.createSequentialGroup()
														.addContainerGap()
														.addComponent(numClustersCb, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
														.addGap(18, 18, 18)
														.addComponent(elbowGraphButton)))
										.addGap(0, 0, Short.MAX_VALUE))
								.addGroup(layout.createSequentialGroup()
										.addContainerGap()
										.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
												.addComponent(heatmapKmeansLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
												.addComponent(clusteringSettingsLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
												.addGroup(layout.createSequentialGroup()
														.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
																.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
																		.addComponent(algorithmCb, javax.swing.GroupLayout.Alignment.LEADING, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
																		.addComponent(clusteringModeCb, javax.swing.GroupLayout.Alignment.LEADING, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
																		.addComponent(clusteringAlgorithmLabel, javax.swing.GroupLayout.Alignment.LEADING))
																.addComponent(clusteringModeLabel)
																.addComponent(parameterValueLabel)
																.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
																		.addGroup(layout.createSequentialGroup()
																				.addComponent(parameterValueSlider, javax.swing.GroupLayout.PREFERRED_SIZE, 160, javax.swing.GroupLayout.PREFERRED_SIZE)
																				.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
																				.addComponent(timeCb, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
																		.addComponent(jSeparator1, javax.swing.GroupLayout.PREFERRED_SIZE, 257, javax.swing.GroupLayout.PREFERRED_SIZE)
																		.addComponent(numClustersLabel))
																.addComponent(heatmapDBSCANLabel)
																.addComponent(epsSlider, javax.swing.GroupLayout.PREFERRED_SIZE, 257, javax.swing.GroupLayout.PREFERRED_SIZE)
																.addComponent(minPointsSlider, javax.swing.GroupLayout.PREFERRED_SIZE, 257, javax.swing.GroupLayout.PREFERRED_SIZE)
																.addComponent(noiseToClusterCheckbox, javax.swing.GroupLayout.PREFERRED_SIZE, 257, javax.swing.GroupLayout.PREFERRED_SIZE))
														.addGap(0, 0, Short.MAX_VALUE))
												.addComponent(jSeparator2)
												.addComponent(jSeparator3))))
						.addContainerGap())
				.addGroup(layout.createSequentialGroup()
						.addGap(67, 67, 67)
						.addComponent(startButton)
						.addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
				);
		layout.setVerticalGroup(
				layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(layout.createSequentialGroup()
						.addGap(24, 24, 24)
						.addComponent(clusteringSettingsLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 32, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(clusteringModeLabel)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(clusteringModeCb, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addGap(18, 18, 18)
						.addComponent(clusteringAlgorithmLabel)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(algorithmCb, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addGap(18, 18, 18)
						.addComponent(parameterValueLabel)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
								.addComponent(parameterValueSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
								.addComponent(timeCb, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
						.addGap(18, 18, 18)
						.addComponent(startClusteringButton)
						.addGap(18, 18, 18)
						.addComponent(jSeparator1, javax.swing.GroupLayout.PREFERRED_SIZE, 10, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(heatmapKmeansLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(numClustersLabel)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
								.addComponent(numClustersCb, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
								.addComponent(elbowGraphButton))
						.addGap(18, 18, 18)
						.addComponent(jSeparator2, javax.swing.GroupLayout.PREFERRED_SIZE, 10, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(heatmapDBSCANLabel)
						.addGap(18, 18, 18)
						.addComponent(minPointsSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addGap(18, 18, 18)
						.addComponent(epsSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addGap(18, 18, 18)
						.addComponent(noiseToClusterCheckbox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addGap(18, 18, 18)
						.addComponent(jSeparator3, javax.swing.GroupLayout.PREFERRED_SIZE, 10, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addGap(18, 18, 18)
						.addComponent(startButton)
						.addContainerGap(50, Short.MAX_VALUE))
				);

		// DBSCAN noise to cluster Checkbox ActionListener
		noiseToClusterCheckbox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				JCheckBox cbNoise = (JCheckBox) evt.getSource();
				if (cbNoise.isSelected()) {
					noiseToClusterFlag = true;
					//					System.out.println("noise to cluster set to TRUE");
				} else {
					noiseToClusterFlag = false;
					//					System.out.println("noise to cluster set to FALSE");
				}
			}
		});

		// Clustering Mode ComboBox Information
		clusteringModeCb.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent evt) {
				JComboBox comboBox = (JComboBox) evt.getSource();
				Object item = evt.getItem();
				if (evt.getStateChange() == ItemEvent.SELECTED) {
					try {
						if (clusteringModeCb.getSelectedItem().equals("Clustering Sessions Based on Time")) {
							if (timeFlag = false) {
								chartPanelTime = createElbowChartTime(sqErrorsTime);
							}
						} else if (clusteringModeCb.getSelectedItem().equals("Clustering Sessions Based on Frequency")) {
							if (countFlag == false) {
								try {
									chartPanelCount = createElbowChartCount(sqErrorsCount);
								} catch (Exception e) {}
							}
						}
					} catch (Exception e) {}
				}
			}
		});

		//		clusterButton = slickerFactory.createButton("Start Clustering");
		startClusteringButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				String timeParameter = (String) timeCb.getSelectedItem();
				globalParameter.setComboBoxItem(timeParameter);

				if (timeParameter.equals("Minutes")) {
					globalParameter.setMineMode(globalParameter.MINUTES);
				} else if (timeParameter.equals("Hours")) {
					globalParameter.setMineMode(globalParameter.HOURS);
				} else if (timeParameter.equals("Days")) {
					globalParameter.setMineMode(globalParameter.DAYS);
				} else if (timeParameter.equals("Weeks")) {
					globalParameter.setMineMode(globalParameter.WEEKS);
				} else if (timeParameter.equals("Months")) {
					globalParameter.setMineMode(globalParameter.MONTHS);
				}

				int integerSliderValue = parameterValueSlider.getValue();
				globalParameter.setTextFieldValue(integerSliderValue);

				if (((String) clusteringModeCb.getSelectedItem()).equals("Encoding Sessions Based on Time")) {
					TimeAndCountPlugin3 plugin = new TimeAndCountPlugin3();
					plugin.setMode((String) clusteringModeCb.getSelectedItem());
					final TimeAndCountModel modelTime = plugin.mineParameters(globalContext, globalModel.getEventLog(), globalParameter);

					// First of all compute the squared errors for the elbow graph
					if (algorithmCb.getSelectedItem().equals("K-Means")) {
						try {
							sqErrorsTime = elbow.run(modelTime.getDataSet());
							chartPanelTime = createElbowChartTime(sqErrorsTime);
						} catch (Exception e1) {}
					}

					timeFlag = true;
				} else if (((String) clusteringModeCb.getSelectedItem()).equals("Encoding Sessions Based on Frequency")) {
					TimeAndCountPlugin3 plugin = new TimeAndCountPlugin3();
					plugin.setMode((String) clusteringModeCb.getSelectedItem());
					final TimeAndCountModel modelCount = plugin.mineParameters(globalContext, globalModel.getEventLog(), globalParameter);

					if (algorithmCb.getSelectedItem().equals("K-Means")) {
						try {
							sqErrorsCount = elbow.run(modelCount.getDataSet());
							chartPanelCount = createElbowChartCount(sqErrorsCount);
						} catch (Exception e2) {}
					}

					countFlag = true;
				} 
			}
		});

		// Number of Cluster Selection Information

		elbowGraphButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				try {
					if (clusteringModeCb.getSelectedItem().equals("Encoding Sessions Based on Time")) {
						chartDialog = new ChartDialog(chartPanelTime);
					} else if (clusteringModeCb.getSelectedItem().equals("Encoding Sessions Based on Frequency")) {
						chartDialog = new ChartDialog(chartPanelCount);
					}
					if (chartDialog != null) {
						chartDialog.setAlwaysOnTop(true);
						chartDialog.setLocation(MouseInfo.getPointerInfo().getLocation());
						chartDialog.setVisible(true);
					}
				} catch (Exception e) {}
			}
		});

		// The Generate Heatmap Button
		//		startButton = new JButton("Generate Heatmap");
		startButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (algorithmCb.getSelectedItem().equals("K-Means")) {
					addATabKmeans();
				} else if (algorithmCb.getSelectedItem().equals("DBSCAN")) {
					addATabDBSCAN();
				}
			}
		});

		// Wrap everything together and return
		entireView.setLeftComponent(leftPane);
		headPanel.add(entireView, BorderLayout.CENTER);

		return headPanel;
	}

	private ChartPanel createElbowChartTime(double[] sqErrorsTime) {
		XYDataset dataset = createDatasetTime();

		JFreeChart chart = ChartFactory.createXYLineChart("Elbow Method Time Based (" + parameterValueSlider.getValue() + " " + timeCb.getSelectedItem().toString() + ")" , 
				"Number of Clusters", "Within Sum of Squared Errors", 
				dataset,
				PlotOrientation.VERTICAL, true, true, false);

		// Customize the chart
		XYPlot plot = chart.getXYPlot();
		XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();

		renderer.setSeriesPaint(0, Color.RED);
		renderer.setSeriesPaint(1, Color.BLACK);
		renderer.setSeriesStroke(0, new BasicStroke(3.0f));

		plot.setRenderer(renderer);
		plot.setBackgroundPaint(Color.DARK_GRAY);
		plot.setRangeGridlinesVisible(true);
		plot.setRangeGridlinePaint(Color.BLACK);
		plot.setDomainGridlinesVisible(true);
		plot.setDomainGridlinePaint(Color.BLACK);

		// Create the ChartPanel
		ChartPanel panel = new ChartPanel(chart);


		return panel;
	}

	private XYDataset createDatasetTime() {
		XYSeriesCollection dataset = new XYSeriesCollection();
		XYSeries series = new XYSeries("Within Sum of Squared Errors");
		for (int i = 0; i < sqErrorsTime.length; i++) {
			series.add(i + 1, sqErrorsTime[i]);
		}

		//		XYSeries series2 = new XYSeries("Straight Line");
		//		series2.add(1, sqErrorsTime[0]);
		//		series2.add(20, sqErrorsTime[19]);

		// Add series to dataset
		dataset.addSeries(series);
		//		dataset.addSeries(series2);

		return dataset;
	}

	private ChartPanel createElbowChartCount(double[] sqErrorsCount) {
		XYDataset dataset = createDatasetCount();

		JFreeChart chart = ChartFactory.createXYLineChart("The Elbow Method Frequency Based (" + parameterValueSlider.getValue() + " " + timeCb.getSelectedItem().toString() + ")", 
				"Number of Clusters", "Within Sum of Squared Errors", 
				dataset,
				PlotOrientation.VERTICAL, true, true, false);

		// Customize the chart
		XYPlot plot = chart.getXYPlot();
		XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();

		renderer.setSeriesPaint(0, Color.RED);
		renderer.setSeriesPaint(1, Color.BLACK);
		renderer.setSeriesStroke(0, new BasicStroke(3.0f));

		plot.setRenderer(renderer);
		plot.setBackgroundPaint(Color.DARK_GRAY);
		plot.setRangeGridlinesVisible(true);
		plot.setRangeGridlinePaint(Color.BLACK);
		plot.setDomainGridlinesVisible(true);
		plot.setDomainGridlinePaint(Color.BLACK);

		// Create the ChartPanel
		ChartPanel panel = new ChartPanel(chart);

		return panel;
	}

	private XYDataset createDatasetCount() {
		XYSeriesCollection dataset = new XYSeriesCollection();
		XYSeries series = new XYSeries("Within Sum of Squared Errors");
		for (int i = 0; i < sqErrorsCount.length; i++) {
			series.add(i + 1, sqErrorsCount[i]);
		}

		//		XYSeries series2 = new XYSeries("Straight Line");
		//		series2.add(1, sqErrorsCount[0]);
		//		series2.add(20, sqErrorsCount[19]);

		// Add series to dataset
		dataset.addSeries(series);
		//		dataset.addSeries(series2);

		return dataset;
	}

	private HeatChart createHeatmapKmeans(KMeans kmeans, String colourScale) {
		heatmap = new HeatChart(kmeans.getMatrix());
		heatmap.setTitle("Clustering Results K-Means");
		//		heatmap.setXValues(kmeans.getAttributeNames().toArray());
		//		heatmap.setYValues(kmeans.getClusters().toArray());
		//		heatmap.setXAxisLabel("Event");
		//		heatmap.setYAxisLabel("cluster");
		heatmap.setXValues(kmeans.getClusters().toArray());
		heatmap.setXAxisLabel("Cluster");
		heatmap.setYValues(kmeans.getAttributeNames().toArray());
		heatmap.setYAxisLabel("Event");

		if (colourScale.equals("Logarithmic")) {
			heatmap.setColourScale(heatmap.SCALE_LOGARITHMIC);
		} else if (colourScale.equals("Linear")) {
			heatmap.setColourScale(heatmap.SCALE_LINEAR);
		} else if (colourScale.equals("Exponential")) {
			heatmap.setColourScale(heatmap.SCALE_EXPONENTIAL);
		}

		heatmap.setHighValueColour(Color.RED);
		return heatmap;
	}

	//		private HeatChart createHeatMapDbscan(Dbscan dbscan, String colourScale) {
	//			heatmap = new HeatChart(dbscan.getMatrix());
	//			heatmap.setTitle("Clustering Results DBSCAN");
	//			heatmap.setXValues(dbscan.getClusters().toArray());
	//			heatmap.setXAxisLabel("Cluster");
	//			heatmap.setYValues(dbscan.getAttributeNames().toArray());
	//			heatmap.setYAxisLabel("Event");
	//	
	//			if (colourScale.equals("Logarithmic")) {
	//				heatmap.setColourScale(heatmap.SCALE_LOGARITHMIC);
	//			} else if (colourScale.equals("Linear")) {
	//				heatmap.setColourScale(heatmap.SCALE_LINEAR);
	//			} else if (colourScale.equals("Exponential")) {
	//				heatmap.setColourScale(heatmap.SCALE_EXPONENTIAL);
	//			}
	//	
	//			heatmap.setHighValueColour(Color.RED);
	//			return heatmap;
	//		}

	private HeatChart createHeatMapDbscan(DbscanElki dbscan, String colourScale) {
		try {
			heatmap = new HeatChart(dbscan.getMatrix());

			heatmap.setTitle("Clustering Results DBSCAN");
			heatmap.setXValues(dbscan.getClusters().toArray());
			heatmap.setXAxisLabel("Cluster");
			heatmap.setYValues(dbscan.getAttributeNames().toArray());
			heatmap.setYAxisLabel("Event");

			if (colourScale.equals("Logarithmic")) {
				heatmap.setColourScale(heatmap.SCALE_LOGARITHMIC);
			} else if (colourScale.equals("Linear")) {
				heatmap.setColourScale(heatmap.SCALE_LINEAR);
			} else if (colourScale.equals("Exponential")) {
				heatmap.setColourScale(heatmap.SCALE_EXPONENTIAL);
			}

			heatmap.setHighValueColour(Color.RED);
		} catch (Exception e) {

		}		

		return heatmap;
	}

	public void addATabKmeans() {
		Executor exec = globalContext.getExecutor();
		Runnable task = new Runnable() {
			public void run() {
				startButton.setText("Generating Heatmap...");

				final Parameters parameters = globalModel.getParameterValues();
				System.out.println(parameters.getTextFieldValue());
				TimeAndCountPlugin3 plugin = new TimeAndCountPlugin3();
				plugin.setMode((String) clusteringModeCb.getSelectedItem());
				final TimeAndCountModel model = plugin.mineParameters(globalContext, globalModel.getEventLog(), parameters);
				final CreateAbstractedLog createAbstractLog = new CreateAbstractedLog();

				final KMeans kmeans = new KMeans();

				int clusterNumber  = (int) numClustersCb.getSelectedItem();
				kmeans.setNumberOfClusters(clusterNumber);
				try {
					kmeans.buildIt(model.getDataSet());
				} catch (Exception e1) {
				}

				// Right TabPane Labels
				final JLabel colourScaleLabel = new JLabel();
				final JLabel thresholdSliderLabel = new JLabel();
				final JLabel clusterModeLabel = new JLabel();
				final JLabel clusterAlgorithmLabel = new JLabel();
				final JLabel numClusterLabel = new JLabel();
				final JLabel timeIntervalLabel = new JLabel();
				final JLabel clusterInformationLabel = new JLabel();
				final JLabel heatmapSettingsLabel = new JLabel();
				// Right TabPane TextFields
				final JTextField textField1 = new JTextField();
				final JTextField textField2 = new JTextField();
				final JTextField textField3 = new JTextField();
				final JTextField textField4 = new JTextField();
				// Right TabPane Other Components
				final JComboBox<String> colourScaleCb = new JComboBox(new String[] {"Logarithmic", "Linear", "Exponential"});
				final NiceDoubleSlider thresholdSliderNice = slickerFactory.createNiceDoubleSlider("", kmeans.getMinValue(), kmeans.getMaxValue(), kmeans.getMinValue(), Orientation.HORIZONTAL);
				final JButton updateButton = new JButton("Update Heatmap");
				final JPanel darkPanel = new JPanel();
				final JSeparator separator = new JSeparator();

				colourScaleLabel.setFont(new Font("Tahoma", 1, 12));
				colourScaleLabel.setText("Heatmap Colouring Scale");

				thresholdSliderLabel.setFont(new Font("Tahoma", 1, 12));
				thresholdSliderLabel.setText("Threshold Value");

				darkPanel.setBackground(Color.GRAY);

				clusterModeLabel.setText("Encoding Mode:");
				textField1.setText((String) clusteringModeCb.getSelectedItem());
				textField1.setEditable(false);
				clusterAlgorithmLabel.setText("Clustering Algorithm:");
				textField2.setText((String) algorithmCb.getSelectedItem());
				textField2.setEditable(false);
				numClusterLabel.setText("Number Of Clusters:");
				textField3.setText(numClustersCb.getSelectedItem().toString());
				textField3.setEditable(false);
				timeIntervalLabel.setText("Time Interval:");
				textField4.setText(parameterValueSlider.getValue() + " " + (String) timeCb.getSelectedItem());
				textField4.setEditable(false);

				javax.swing.GroupLayout darkPanelLayout = new javax.swing.GroupLayout(darkPanel);
				darkPanel.setLayout(darkPanelLayout);
				darkPanelLayout.setHorizontalGroup(
						darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(darkPanelLayout.createSequentialGroup()
								.addContainerGap()
								.addGroup(darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
										.addComponent(clusterAlgorithmLabel)
										.addComponent(numClusterLabel)
										.addComponent(clusterModeLabel)
										.addComponent(timeIntervalLabel))
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addGroup(darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
										.addComponent(textField3)
										.addComponent(textField2)
										.addComponent(textField1)
										.addComponent(textField4))
								.addContainerGap())
						);
				darkPanelLayout.setVerticalGroup(
						darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(darkPanelLayout.createSequentialGroup()
								.addContainerGap()
								.addGroup(darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
										.addComponent(clusterModeLabel)
										.addComponent(textField1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addGroup(darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
										.addComponent(clusterAlgorithmLabel)
										.addComponent(textField2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addGroup(darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
										.addComponent(numClusterLabel)
										.addComponent(textField3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
								.addGap(7, 7, 7)
								.addGroup(darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
										.addComponent(timeIntervalLabel)
										.addComponent(textField4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
								.addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
						);

				clusterInformationLabel.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 18));
				clusterInformationLabel.setText("Clustering Information");
				heatmapSettingsLabel.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 18));
				heatmapSettingsLabel.setText("Heatmap Settings");

				final JButton exportButton = new JButton("Generate Abstracted Log");
				//				final JButton exportButton = slickerFactory.createButton("Generate Abstracted Log");
				//				final JCheckBox checkbox = slickerFactory.createCheckBox("Enable Numbering", false);
				final JCheckBox checkbox = new JCheckBox("Enable Numbering", false);

				JPanel rightTabPane = new JPanel();
				rightTabPane.setPreferredSize(new Dimension(250, 80));
				rightTabPane.setBackground(Color.LIGHT_GRAY);
				rightTabPane.setForeground(Color.BLACK);

				javax.swing.GroupLayout layout = new javax.swing.GroupLayout(rightTabPane);
				rightTabPane.setLayout(layout);
				layout.setHorizontalGroup(
						layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(layout.createSequentialGroup()
								.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
										.addGroup(layout.createSequentialGroup()
												.addContainerGap()
												.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(darkPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
														.addComponent(separator)
														.addGroup(layout.createSequentialGroup()
																.addComponent(thresholdSliderLabel)
																.addGap(18, 18, 18)
																.addComponent(thresholdSliderNice, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
														.addGroup(layout.createSequentialGroup()
																.addComponent(colourScaleLabel)
																.addGap(18, 18, 18)
																.addComponent(colourScaleCb, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
										.addGroup(layout.createSequentialGroup()
												.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
														.addGroup(layout.createSequentialGroup()
																.addGap(93, 93, 93)
																.addComponent(clusterInformationLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
														.addGroup(layout.createSequentialGroup()
																.addGap(120, 120, 120)
																.addComponent(heatmapSettingsLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
																.addGap(20, 20, 20)))
												.addGap(88, 88, 88)))
								.addContainerGap())
						.addGroup(layout.createSequentialGroup()
								.addGap(100, 100, 100)
								.addComponent(updateButton, javax.swing.GroupLayout.PREFERRED_SIZE, 200, javax.swing.GroupLayout.PREFERRED_SIZE)
								.addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
						);
				layout.setVerticalGroup(
						layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(layout.createSequentialGroup()
								.addContainerGap()
								.addComponent(clusterInformationLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(darkPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
								.addGap(18, 18, 18)
								.addComponent(separator, javax.swing.GroupLayout.PREFERRED_SIZE, 10, javax.swing.GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(heatmapSettingsLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
								.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
										.addComponent(colourScaleLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
										.addComponent(colourScaleCb, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
								.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
										.addComponent(thresholdSliderLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
										.addComponent(thresholdSliderNice, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
								.addGap(45, 45, 45)
								.addComponent(updateButton)
								.addGap(346, 346, 346))
						);


				String cs = (String) colourScaleCb.getSelectedItem();
				heatmap = createHeatmapKmeans(kmeans, cs);
				icon = new ImageIcon(heatmap.getChartImage());
				imageLabel = new JLabel(icon);

				JPanel clusterPanel = new JPanel();
				clusterPanel.setLayout(new BoxLayout(clusterPanel, BoxLayout.LINE_AXIS));
				JPanel clusterNamesPanel = new JPanel();
				clusterNamesPanel.setLayout(new GridLayout(kmeans.getNumberOfClusters(), 2));
				final List<JTextField> textFields = new ArrayList<JTextField>();
				for (int i = 0; i < kmeans.getNumberOfClusters(); i++) {
					JLabel label = new JLabel("Cluster " + i + ": ");
					JTextField textField = new JTextField(20);
					label.setLabelFor(textField);
					clusterNamesPanel.add(label);
					clusterNamesPanel.add(textField);
					textFields.add(textField);
				}
				clusterPanel.add(clusterNamesPanel);

				// Insert margin
				clusterPanel.add(Box.createRigidArea(new Dimension(0, 15)));

				JPanel checkboxExportButtonPanel = new JPanel();
				Container c = new Container();
				c.setLayout(new BoxLayout(c, BoxLayout.PAGE_AXIS));
				c.add(checkbox);
				c.add(exportButton);
				checkboxExportButtonPanel.add(c);
				clusterPanel.add(checkboxExportButtonPanel);

				// Pre-fill the textfields with the name of the events with the highest value in each of the clusters
				ArrayList<String> maxEventNames = new ArrayList<String>();
				for (Integer cluster : kmeans.getClustertoAttributeValueMap().keySet()) {
					Map<String, Double> map = kmeans.getClustertoAttributeValueMap().get(cluster);
					double maxValueInMap = (Collections.max(map.values()));
					for (Entry<String, Double> entry : map.entrySet()) {
						if (entry.getValue() == maxValueInMap) {
							maxEventNames.add(entry.getKey());
						}
					}
				}            

				int i = 0;
				for (JTextField tf: textFields) {
					tf.setText(maxEventNames.get(i));
					i++;
				}

				JScrollPane heatmapScrollPane = new JScrollPane(imageLabel);
				final JSplitPane resultSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, heatmapScrollPane, clusterPanel);
				resultSplitPane.setResizeWeight(0.85);
				JSplitPane tabViewPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, resultSplitPane, rightTabPane);
				tabViewPane.setOpaque(false);
				tabViewPane.setOneTouchExpandable(true);
				tabViewPane.setResizeWeight(1);

				ButtonTabComponent ctb = new ButtonTabComponent(tabbedPane);
				if (clusteringModeCb.getSelectedItem().equals("Encoding Sessions Based on Frequency")) {					
					if (!isTabExist(kmeans.getNumberOfClusters() + " clusters (count based)")) {
						tabbedPane.insertTab(kmeans.getNumberOfClusters() + " clusters (count based)", null, tabViewPane, kmeans.getNumberOfClusters() + " clusters (count based)", numberOfTabs);
						tabbedPane.setTabComponentAt(numberOfTabs, ctb);
						tabbedPane.setSelectedIndex(numberOfTabs);
						numberOfTabs = tabbedPane.getTabCount();
					}
				} else {
					if (!isTabExist(kmeans.getNumberOfClusters() + " clusters (time based)")) {
						tabbedPane.insertTab(kmeans.getNumberOfClusters() + " clusters (time based)", null, tabViewPane, kmeans.getNumberOfClusters() + " clusters (time based)", numberOfTabs);
						tabbedPane.setTabComponentAt(numberOfTabs, ctb);
						tabbedPane.setSelectedIndex(numberOfTabs);
						numberOfTabs = tabbedPane.getTabCount();
					}
				}

				numberOfTabs = ctb.getTabCount();
				startButton.setText("Generate Heatmap");

				updateButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent evt) {
						try {
							kmeans.setActualThresholdValue(thresholdSliderNice.getValue());
							kmeans.runFiltered(model.getDataSet());
						} catch (Exception e) {
						}
						String cs = (String) colourScaleCb.getSelectedItem();
						heatmap = createHeatmapKmeans(kmeans, cs);
						icon = new ImageIcon(heatmap.getChartImage());
						imageLabel = new JLabel(icon);
						JScrollPane heatmapScrollPane = new JScrollPane(imageLabel);
						resultSplitPane.setLeftComponent(heatmapScrollPane);
					}
				});

				exportButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						exportButton.setText("Creating Abstracted Log...");

						clusterNameMap = new TreeMap<Integer, String>();
						for (int i = 0; i < textFields.size(); i++) {
							clusterNameMap.put(i, textFields.get(i).getText());
						}
						globalModel.setClusterNameMap(clusterNameMap);

						int[] assignments = kmeans.getClusterAssignments();
						indexToClusterMap = new TreeMap<Integer, Integer>();
						for (int i = 0; i < assignments.length; i++) {
							indexToClusterMap.put(i, assignments[i]);
						}
						globalModel.setIndexToClusterMap(indexToClusterMap);

						if (countFlag == true) {
							if (publishNumberedFlag == true) {
								if (parameters.getMineMode() == parameters.MINUTES) {
									XLog abstractLog = createAbstractLog.mineAbstractLogMinutes(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Numbered Abstract Log Minutes Count (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.HOURS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogHours(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Numbered Abstract Log Hours Count (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.DAYS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogDays(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Numbered Abstract Log Days Count (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.WEEKS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogWeeks(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Numbered Abstract Log Weeks Count (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.MONTHS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogMonths(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Numbered Abstract Log Months Count (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								}
							} else {
								if (parameters.getMineMode() == parameters.MINUTES) {
									XLog abstractLog = createAbstractLog.mineAbstractLogMinutes(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Abstract Log Minutes Count (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.HOURS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogHours(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Abstract Log Hours Count (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.DAYS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogDays(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Abstract Log Days Count (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.WEEKS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogWeeks(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Abstract Log Weeks Count (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.MONTHS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogMonths(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Abstract Log Months Count (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								}
							}
						} else {
							if (publishNumberedFlag == true) {
								if (parameters.getMineMode() == parameters.MINUTES) {
									XLog abstractLog = createAbstractLog.mineAbstractLogMinutes(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Numbered Abstract Log Minutes (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.HOURS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogHours(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Numbered Abstract Log Hours (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.DAYS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogDays(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Numbered Abstract Log Days (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.WEEKS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogWeeks(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Numbered Abstract Log Weeks (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.MONTHS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogMonths(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Numbered Abstract Log Months (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								}
							} else {
								if (parameters.getMineMode() == parameters.MINUTES) {
									XLog abstractLog = createAbstractLog.mineAbstractLogMinutes(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Abstract Log Minutes (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.HOURS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogHours(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Abstract Log Hours (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.DAYS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogDays(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Abstract Log Days (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.WEEKS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogWeeks(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Abstract Log Days (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								} else if (parameters.getMineMode() == parameters.MONTHS) {
									XLog abstractLog = createAbstractLog.mineAbstractLogMonths(globalModel.getEventLog(), globalModel);
									publish(globalContext, "Abstract Log Months (" + kmeans.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
								}
							}
						}

						exportButton.setText("Abstracted Log Created");
						try {
							Thread.currentThread().sleep(2000);
						} catch (InterruptedException e1) {
						}      

						Timer timer2 = new Timer(2000, new ActionListener() {
							public void actionPerformed(ActionEvent e) {
								exportButton.setText("Generate Abstracted Log");
							}
						});
						timer2.start();

					}
				});

				checkbox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent evt) {
						JCheckBox cbLog = (JCheckBox) evt.getSource();
						if (cbLog.isSelected()) {
							createAbstractLog.setWithNumbers(true);
							publishNumberedFlag = true;
						} else {
							createAbstractLog.setWithNumbers(false);
							publishNumberedFlag = false;
						}
					}       
				});

				startButton.setText("Generate Heatmap");
			}
		};
		exec.execute(task);

	} // End of addATabKmeans method

	public void addATabDBSCAN() {
		Executor exec = globalContext.getExecutor();
		//		Runnable task = new Runnable() {
		//			public void run() {
		startButton.setText("Generating Heatmap (DBSCAN)...");

		final Parameters parameters = globalModel.getParameterValues();
		TimeAndCountPlugin3 plugin = new TimeAndCountPlugin3();
		plugin.setMode((String) clusteringModeCb.getSelectedItem());
		final TimeAndCountModel model = plugin.mineParameters(globalContext, globalModel.getEventLog(), parameters);
		final CreateAbstractedLog createAbstractLog = new CreateAbstractedLog();

		//						final Dbscan dbscan = new Dbscan();

		DbscanElki dbscan = new DbscanElki();

		dbscan.setWithNoiseToCluster(noiseToClusterFlag);
		dbscan.setEpsilon(epsSlider.getValue());
		dbscan.setMinPoints(minPointsSlider.getValue());

		try {
			dbscan.buildIt(model.getElkiDataSet(), model.getLabels());
			//								dbscan.buildIt(model.getDataSet());
		} catch (Exception e) {
			System.out.println("No Clusters found, try different parameters!");
		}

		// Right TabPane Labels
		final JLabel colourScaleLabel = new JLabel();
		final JLabel thresholdSliderLabel = new JLabel();
		final JLabel clusterModeLabel = new JLabel();
		final JLabel clusterAlgorithmLabel = new JLabel();
		final JLabel numClusterLabel = new JLabel();
		final JLabel timeIntervalLabel = new JLabel();
		final JLabel clusterInformationLabel = new JLabel();
		final JLabel heatmapSettingsLabel = new JLabel();
		// Right TabPane TextFields
		final JTextField textField1 = new JTextField();
		final JTextField textField2 = new JTextField();
		final JTextField textField3 = new JTextField();
		final JTextField textField4 = new JTextField();
		// Right TabPane Other Components
		final JComboBox<String> colourScaleCb = new JComboBox(new String[] {"Logarithmic", "Linear", "Exponential"});
		final NiceDoubleSlider thresholdSliderNice = slickerFactory.createNiceDoubleSlider("", dbscan.getMinValue(), dbscan.getMaxValue(), dbscan.getMinValue(), Orientation.HORIZONTAL);
		final JButton updateButton = new JButton("Update Heatmap");
		final JPanel darkPanel = new JPanel();
		final JSeparator separator = new JSeparator();

		colourScaleLabel.setFont(new Font("Tahoma", 1, 12));
		colourScaleLabel.setText("Heatmap Colouring Scale");

		thresholdSliderLabel.setFont(new Font("Tahoma", 1, 12));
		thresholdSliderLabel.setText("Threshold Value");

		darkPanel.setBackground(Color.GRAY);

		clusterModeLabel.setText("Encoding Mode:");
		textField1.setText((String) clusteringModeCb.getSelectedItem());
		textField1.setEditable(false);
		clusterAlgorithmLabel.setText("Clustering Algorithm:");
		textField2.setText((String) algorithmCb.getSelectedItem());
		textField2.setEditable(false);
		numClusterLabel.setText("Number Of Clusters:");
		textField3.setText(Integer.toString(dbscan.getNumberOfClusters()));
		textField3.setEditable(false);
		timeIntervalLabel.setText("Time Interval:");
		textField4.setText(parameterValueSlider.getValue() + " " + (String) timeCb.getSelectedItem());
		textField4.setEditable(false);

		javax.swing.GroupLayout darkPanelLayout = new javax.swing.GroupLayout(darkPanel);
		darkPanel.setLayout(darkPanelLayout);
		darkPanelLayout.setHorizontalGroup(
				darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(darkPanelLayout.createSequentialGroup()
						.addContainerGap()
						.addGroup(darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
								.addComponent(clusterAlgorithmLabel)
								.addComponent(numClusterLabel)
								.addComponent(clusterModeLabel)
								.addComponent(timeIntervalLabel))
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addGroup(darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
								.addComponent(textField3)
								.addComponent(textField2)
								.addComponent(textField1)
								.addComponent(textField4))
						.addContainerGap())
				);
		darkPanelLayout.setVerticalGroup(
				darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(darkPanelLayout.createSequentialGroup()
						.addContainerGap()
						.addGroup(darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
								.addComponent(clusterModeLabel)
								.addComponent(textField1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addGroup(darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
								.addComponent(clusterAlgorithmLabel)
								.addComponent(textField2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addGroup(darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
								.addComponent(numClusterLabel)
								.addComponent(textField3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
						.addGap(7, 7, 7)
						.addGroup(darkPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
								.addComponent(timeIntervalLabel)
								.addComponent(textField4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
						.addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
				);

		clusterInformationLabel.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 18));
		clusterInformationLabel.setText("Clustering Information");
		heatmapSettingsLabel.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 18));
		heatmapSettingsLabel.setText("Heatmap Settings");

		final JButton exportButton = new JButton("Generate Abstracted Log");
		//				final JButton exportButton = slickerFactory.createButton("Generate Abstracted Log");
		//				final JCheckBox checkbox = slickerFactory.createCheckBox("Enable Numbering", false);
		final JCheckBox checkbox = new JCheckBox("Enable Numbering", false);

		JPanel rightTabPane = new JPanel();
		rightTabPane.setPreferredSize(new Dimension(250, 80));
		rightTabPane.setBackground(Color.LIGHT_GRAY);
		rightTabPane.setForeground(Color.BLACK);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(rightTabPane);
		rightTabPane.setLayout(layout);
		layout.setHorizontalGroup(
				layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(layout.createSequentialGroup()
						.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
								.addGroup(layout.createSequentialGroup()
										.addContainerGap()
										.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
												.addComponent(darkPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
												.addComponent(separator)
												.addGroup(layout.createSequentialGroup()
														.addComponent(thresholdSliderLabel)
														.addGap(18, 18, 18)
														.addComponent(thresholdSliderNice, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
												.addGroup(layout.createSequentialGroup()
														.addComponent(colourScaleLabel)
														.addGap(18, 18, 18)
														.addComponent(colourScaleCb, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
								.addGroup(layout.createSequentialGroup()
										.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
												.addGroup(layout.createSequentialGroup()
														.addGap(93, 93, 93)
														.addComponent(clusterInformationLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
												.addGroup(layout.createSequentialGroup()
														.addGap(120, 120, 120)
														.addComponent(heatmapSettingsLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
														.addGap(20, 20, 20)))
										.addGap(88, 88, 88)))
						.addContainerGap())
				.addGroup(layout.createSequentialGroup()
						.addGap(100, 100, 100)
						.addComponent(updateButton, javax.swing.GroupLayout.PREFERRED_SIZE, 200, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
				);
		layout.setVerticalGroup(
				layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(layout.createSequentialGroup()
						.addContainerGap()
						.addComponent(clusterInformationLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(darkPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
						.addGap(18, 18, 18)
						.addComponent(separator, javax.swing.GroupLayout.PREFERRED_SIZE, 10, javax.swing.GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(heatmapSettingsLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
						.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
								.addComponent(colourScaleLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
								.addComponent(colourScaleCb, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
						.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
								.addComponent(thresholdSliderLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
								.addComponent(thresholdSliderNice, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
						.addGap(45, 45, 45)
						.addComponent(updateButton)
						.addGap(346, 346, 346))
				);


		String cs = (String) colourScaleCb.getSelectedItem();
		if (!dbscan.getClusters().isEmpty()) {
			heatmap = createHeatMapDbscan(dbscan, cs);
			icon = new ImageIcon(heatmap.getChartImage());
			imageLabel = new JLabel(icon);
		}

		if (!dbscan.getClusters().isEmpty()) {
			JPanel clusterPanel = new JPanel();
			clusterPanel.setLayout(new BoxLayout(clusterPanel, BoxLayout.LINE_AXIS));
			JPanel clusterNamesPanel = new JPanel();
			clusterNamesPanel.setLayout(new GridLayout(dbscan.getNumberOfClusters(), 2));
			//				final List<JTextField> textFields = new ArrayList<JTextField>();
			final Map<Integer, JTextField> textFieldsMap = new HashMap<Integer, JTextField>();
			for (int i : dbscan.getClusters()) {
				JLabel label = new JLabel("Cluster " + i + ": ");
				JTextField textField = new JTextField(20);
				label.setLabelFor(textField);
				clusterNamesPanel.add(label);
				clusterNamesPanel.add(textField);
				textFieldsMap.put(i, textField);
				//					textFields.add(textField);
			}
			clusterPanel.add(clusterNamesPanel);

			// Insert margin
			clusterPanel.add(Box.createRigidArea(new Dimension(0, 15)));

			JPanel checkboxExportButtonPanel = new JPanel();
			Container c = new Container();
			c.setLayout(new BoxLayout(c, BoxLayout.PAGE_AXIS));
			c.add(checkbox);
			c.add(exportButton);
			checkboxExportButtonPanel.add(c);
			clusterPanel.add(checkboxExportButtonPanel);

			// Pre-fill the textfields with the name of the events with the highest value in each of the clusters
			ArrayList<String> maxEventNames = new ArrayList<String>();
			for (Integer cluster : dbscan.getClusterToAttributeValueMap().keySet()) {
				if (cluster == dbscan.getNoiseCluster()) {
					maxEventNames.add("NOISE");
				} else {
					Map<String, Double> map = dbscan.getClusterToAttributeValueMap().get(cluster);
					double maxValueInMap = (Collections.max(map.values()));
					for (Entry<String, Double> entry : map.entrySet()) {
						if (entry.getValue() == maxValueInMap) {
							maxEventNames.add(entry.getKey());
						}
					}
				}
			}

			int i = 0;
			for (JTextField tf : textFieldsMap.values()) {
				tf.setText(maxEventNames.get(i));
				i++;
			}

			JScrollPane heatmapScrollPane = new JScrollPane(imageLabel);
			final JSplitPane resultSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, heatmapScrollPane, clusterPanel);
			resultSplitPane.setResizeWeight(0.85);
			JSplitPane tabViewPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, resultSplitPane, rightTabPane);
			tabViewPane.setOpaque(false);
			tabViewPane.setOneTouchExpandable(true);
			tabViewPane.setResizeWeight(1);

			ButtonTabComponent ctb = new ButtonTabComponent(tabbedPane);
			if (clusteringModeCb.getSelectedItem().equals("Encoding Sessions Based on Frequency")) {

				if (!isTabExist(dbscan.getNumberOfClusters() + " clusters (count based)")) {
					tabbedPane.insertTab(dbscan.getNumberOfClusters() + " clusters (count based)", null, tabViewPane, dbscan.getNumberOfClusters() + " clusters (count based)", numberOfTabs);
					tabbedPane.setTabComponentAt(numberOfTabs, ctb);
					tabbedPane.setSelectedIndex(numberOfTabs);
					numberOfTabs = tabbedPane.getTabCount();
				}
			} else {
				if (!isTabExist(dbscan.getNumberOfClusters() + " clusters (time based)")) {
					tabbedPane.insertTab(dbscan.getNumberOfClusters() + " clusters (time based)", null, tabViewPane, dbscan.getNumberOfClusters() + " clusters (time based)", numberOfTabs);
					tabbedPane.setTabComponentAt(numberOfTabs, ctb);
					tabbedPane.setSelectedIndex(numberOfTabs);
					numberOfTabs = tabbedPane.getTabCount();
				}
			}

			numberOfTabs = ctb.getTabCount();
			startButton.setText("Generate Heatmap");

			updateButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent evt) {
					try {
						dbscan.setActualThresholdValue(thresholdSliderNice.getValue());
						dbscan.runFiltered(dbscan.getOldMatrix());
					} catch (Exception e) {
					}
					String cs = (String) colourScaleCb.getSelectedItem();
					heatmap = createHeatMapDbscan(dbscan, cs);
					icon = new ImageIcon(heatmap.getChartImage());
					imageLabel = new JLabel(icon);
					JScrollPane heatmapScrollPane = new JScrollPane(imageLabel);
					resultSplitPane.setLeftComponent(heatmapScrollPane);
				}
			});

			exportButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					exportButton.setText("Creating Abstracted Log...");

					clusterNameMap = new TreeMap<Integer, String>();
					for (Entry<Integer, JTextField> entry : textFieldsMap.entrySet()) {
						clusterNameMap.put(entry.getKey(), textFieldsMap.get(entry.getKey()).getText());
					}
					globalModel.setClusterNameMap(clusterNameMap);

					Map<Integer, Integer> clusterAssignment = dbscan.getClusterAssignments();
					indexToClusterMap = new TreeMap<Integer, Integer>();
					for (Integer i : clusterAssignment.keySet()) {
						indexToClusterMap.put(i, clusterAssignment.get(i));
					}

					globalModel.setIndexToClusterMap(indexToClusterMap);
					//						
					if (countFlag == true) {
						if (publishNumberedFlag == true) {
							if (parameters.getMineMode() == parameters.MINUTES) {
								XLog abstractLog = createAbstractLog.mineAbstractLogMinutes(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Numbered Abstract Log Minutes DBSCAN Count (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.HOURS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogHours(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Numbered Abstract Log Hours DBSCAN Count (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.DAYS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogDays(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Numbered Abstract Log Days Count (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.WEEKS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogWeeks(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Numbered Abstract Log Weeks DBSCAN Count (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.MONTHS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogMonths(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Numbered Abstract Log Months DBSCAN Count (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							}
						} else {
							if (parameters.getMineMode() == parameters.MINUTES) {
								XLog abstractLog = createAbstractLog.mineAbstractLogMinutes(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Abstract Log Minutes DBSCAN Count (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.HOURS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogHours(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Abstract Log Hours DBSCAN Count (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.DAYS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogDays(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Abstract Log Days Count (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.WEEKS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogWeeks(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Abstract Log Weeks DBSCAN Count (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.MONTHS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogMonths(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Abstract Log Months DBSCAN Count (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							}
						}
					} else {
						if (publishNumberedFlag == true) {
							if (parameters.getMineMode() == parameters.MINUTES) {
								XLog abstractLog = createAbstractLog.mineAbstractLogMinutes(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Numbered Abstract Log Minutes DBSCAN (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.HOURS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogHours(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Numbered Abstract Log Hours DBSCAN (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.DAYS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogDays(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Numbered Abstract Log Days (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.WEEKS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogWeeks(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Numbered Abstract Log Weeks DBSCAN (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.MONTHS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogMonths(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Numbered Abstract Log Months DBSCAN (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							}
						} else {
							if (parameters.getMineMode() == parameters.MINUTES) {
								XLog abstractLog = createAbstractLog.mineAbstractLogMinutes(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Abstract Log Minutes DBSCAN (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.HOURS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogHours(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Abstract Log Hours DBSCAN (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.DAYS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogDays(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Abstract Log Days (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.WEEKS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogWeeks(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Abstract Log Weeks DBSCAN (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							} else if (parameters.getMineMode() == parameters.MONTHS) {
								XLog abstractLog = createAbstractLog.mineAbstractLogMonths(globalModel.getEventLog(), globalModel);
								publish(globalContext, "Abstract Log Months DBSCAN (" + dbscan.getNumberOfClusters() + " clusters)", abstractLog, XLog.class, true);
							}
						}
					}
					exportButton.setText("Generate Abstracted Log");
				} 					
			});

			checkbox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent evt) {
					JCheckBox cbLog = (JCheckBox) evt.getSource();
					if (cbLog.isSelected()) {
						createAbstractLog.setWithNumbers(true);
						publishNumberedFlag = true;
					} else {
						createAbstractLog.setWithNumbers(false);
						publishNumberedFlag = false;
					}
				}       
			});
		}
		startButton.setText("Generate Heatmap");
		//			}
		//		};
		//		exec.execute(task);
	} // End of addATabDBSCAN method

	private static <T> void publish(final PluginContext context, final String name, final T object,
			final Class<? super T> clazz, final boolean favorite) {
		final ProvidedObjectID id = context.getProvidedObjectManager().createProvidedObject(name, object, clazz, context);

		if (context instanceof UIPluginContext) {
			final GlobalContext gcontext = ((UIPluginContext) context).getGlobalContext();
			if (gcontext instanceof UIContext) {
				final UIContext uicontext = (UIContext) gcontext;
				final ResourceType resType = uicontext.getResourceManager().getResourceTypeFor(clazz);
				if (resType != null) {
					ProMTask task = null;
					try {
						final Field taskField = context.getClass().getDeclaredField("task");
						taskField.setAccessible(true);
						task = (ProMTask) taskField.get(context);
					} catch (final Exception _) {
						// Guess it wasn't meant to be, then...
					}
					final List<Collection<ProMPOResource>> lst = Collections.emptyList();
					ProMPOResource res = new ProMPOResource(uicontext, task == null ? null : task.getAction(), resType, 
							id, lst);
					res = uicontext.getResourceManager().addResource(id, res);
				}
			}
		}
		if (favorite) {
			ProvidedObjectHelper.setFavorite(context, object);
		}
	} // End of publish method

	private boolean isTabExist(String title) {
		for (int i = 0; i < numberOfTabs; i++) {
			if (tabbedPane.getTitleAt(i).equalsIgnoreCase(title)) {
				tabbedPane.setSelectedIndex(i);
				return true;
			}
		}
		return false;
	}
}
